

import numpy as np
from itertools import permutations 
from util_functions.melodic_components import chord_data
from AuToTen.setup import a_calculator2, common
import math
import statsmodels.api as sm
from pandas import DataFrame   

def block_arrangement(chord_labels, keys_labels, chord_seq):
    
    # Calculating chordal notes
    chordal_notes = []
    scales = []
    melodic_ranges = []
    for idx in range(len(chord_labels)):   
        (current_chordal_notes, 
         scale, 
         current_melodic_range) = chord_data(keys_labels[idx], 
                                             chord_labels[idx], 
                                             (2,3,4))

        chordal_notes.append(current_chordal_notes)
        scales.append(scale)
        melodic_ranges.append(current_melodic_range)

    # Imposed bass line
    root_position = ['first', 'norm', 'dom', 'last']
    bass_line = []
    for chord in range(len(chord_labels)):
        if chord_seq[chord] in root_position:
            bass_line.append(chordal_notes[chord][0])
        elif chord_seq[chord] == 'cad64':
            bass_line.append(chordal_notes[chord][2])
        else:
            bass_line.append(chordal_notes[chord][1])

    # First chord
    
    first_bass = chordal_notes[0][0]+str(3)
    first_tenor = chordal_notes[0][2]+str(3)
    if (melodic_ranges[0].index(first_tenor) 
       < melodic_ranges[0].index(first_bass)):
        first_bass = chordal_notes[0][0]+str(2)
    first_alto = chordal_notes[0][1]+str(4)
    first_soprano = chordal_notes[0][2]+str(4)   
    
    arrangement = [[first_bass, first_tenor, first_alto, first_soprano]]
    previous_arrangement = [first_bass, first_tenor, first_alto, first_soprano]
    previous_scale = scales[0]
    previous_range = melodic_ranges[0]
    
    for chord in range(1, len(chord_labels)):
        # Calculating possible STA arrangements
        possible_chordal_notes = [note+str(octs) 
                                  for octs in range(2,5)
                                  for note in chordal_notes[chord]]
        possible_arrangements = [list(tup)
                                 for tup in list(permutations(possible_chordal_notes, 4)) 
                                if tup[0][:-1] == bass_line[chord]] 
        
        # Calculating intervals between possible voicings
        previous_intervals = interval_calculator(previous_arrangement, 
                                                 previous_scale)
        current_intervals = [interval_calculator(current_arrangement, 
                                                 scales[chord]) 
                            for current_arrangement in possible_arrangements]

        # Delete from possible arrangements:

        # 1) those with parallel 5ths and 8ths
        not_parallel = []
        for arran in range(len(possible_arrangements)):
            parallel = False
            for idx in range(len(current_intervals[arran])):
                if (previous_intervals[idx] 
                    == current_intervals[arran][idx] 
                    == 0 
                    or previous_intervals[idx] 
                    == current_intervals[arran][idx] 
                    == 4):
                    parallel = True
            if not parallel:
                not_parallel.append(possible_arrangements[arran])

        # 2) those where there are distances greater than an 8ve
        within_octave = [arran 
                         for arran in not_parallel 
                         if all(dist <= 7 
                                for dist in intravoice_distance(arran, 
                                                                melodic_ranges[chord]))]

        # 3) those where voices cross each other
        not_crossing = []
        not_crossing_duplicates = []
        for arran in within_octave:
            bass_idx = melodic_ranges[chord].index(arran[0])
            tenor_idx = melodic_ranges[chord].index(arran[1])
            alto_idx = melodic_ranges[chord].index(arran[2])        
            soprano_idx = melodic_ranges[chord].index(arran[3])

            if (soprano_idx > alto_idx > tenor_idx > bass_idx):
                not_crossing.append(arran)
            if (soprano_idx >= alto_idx >= tenor_idx >= bass_idx):
                not_crossing_duplicates.append(arran)               
        if len(not_crossing) == 0:
            not_crossing = not_crossing_duplicates

        # Select final voicing:
        # that with the smallest distance between voices

        possible_distances = [intervoice_distance(previous_range, 
                                                  previous_arrangement, 
                                                  melodic_ranges[chord], 
                                                  current_arrangement) 
                             for current_arrangement in not_crossing]
        if len(possible_distances) == 0:
            arrangement = np.random.choice(possible_arrangements)
        else:
            final_voicing = not_crossing[possible_distances.index(min(possible_distances))]
            arrangement.append(final_voicing)

        previous_arrangement = final_voicing
        previous_scale = scales[chord]
        previous_range = melodic_ranges[chord]    
    
    return arrangement

def interval_calculator(current_arrangement, current_scale):
    
    bass_tenor = abs(current_scale.index(current_arrangement[0][:-1])
                     - current_scale.index(current_arrangement[1][:-1]))%7
    bass_alto = abs(current_scale.index(current_arrangement[0][:-1])
                    - current_scale.index(current_arrangement[2][:-1]))%7
    bass_soprano = abs(current_scale.index(current_arrangement[0][:-1])
                       - current_scale.index(current_arrangement[3][:-1]))%7
    tenor_alto = abs(current_scale.index(current_arrangement[1][:-1])
                     - current_scale.index(current_arrangement[2][:-1]))%7
    tenor_soprano = abs(current_scale.index(current_arrangement[1][:-1])
                        - current_scale.index(current_arrangement[3][:-1]))%7
    alto_soprano = abs(current_scale.index(current_arrangement[2][:-1])
                       - current_scale.index(current_arrangement[3][:-1]))%7
    
    return [bass_tenor,
            bass_alto,
            bass_soprano,
            tenor_alto,
            tenor_soprano,
            alto_soprano]

def intravoice_distance(current_arrangement, 
                        current_range):
    
    bass_tenor = abs(current_range.index(current_arrangement[0])
                     - current_range.index(current_arrangement[1]))
    tenor_alto = abs(current_range.index(current_arrangement[1])
                     - current_range.index(current_arrangement[2]))
    alto_soprano = abs(current_range.index(current_arrangement[2])
                       - current_range.index(current_arrangement[3]))
    
    return [bass_tenor,
            tenor_alto,
            alto_soprano]    

def intervoice_distance(previous_range, 
                        previous_arrangement, 
                        current_range, 
                        current_arrangement):
    
    bass_bass = abs(previous_range.index(previous_arrangement[0]) 
                    - current_range.index(current_arrangement[0]))
    tenor_tenor = abs(previous_range.index(previous_arrangement[1]) 
                      - current_range.index(current_arrangement[1]))
    alto_alto = abs(previous_range.index(previous_arrangement[2]) 
                    - current_range.index(current_arrangement[2]))
    soprano_soprano = abs(previous_range.index(previous_arrangement[3]) 
                          - current_range.index(current_arrangement[3]))
    
    return [bass_bass, 
           tenor_tenor,
           alto_alto,
           soprano_soprano]
