

import numpy as np
from util_parameters.melodic_param import possible_octaves
from util_functions.melodic_sequence import  new_subordinate
from util_functions.melodic_transformations import (melodic_inversion,
                                                    melodic_transposition, 
                                                    inverted_consequence, 
                                                    last_chord, 
                                                    melodic_retrogression)
from util_functions.pb_melody import lasting_bars, theme_optimization, alter_up
from util_functions.melodic_rhythm import rhythm_assignment

def cs_theme(final_cs, 
             final_pb, 
             key, 
             sub_subdoms,
             subdom, 
             cad, 
             dom, 
             last, 
             time_sig, 
             character, 
             first_melodic_theme, 
             first_melodic_rhythm, 
             other_melodic_theme, 
             other_melodic_rhythm, 
             rep_melodic_theme,
             rep_melodic_rhythm,
             first, 
             other, 
             rep,
             strong_beats, 
             reference_dur):
    
     
    # Generating "dom"
    reference_chord = first
    ref_melodic_theme = first_melodic_theme
    ref_melodic_rhythm = first_melodic_rhythm

    (dom_melodic_theme, 
     dom_melodic_rhythm) = inverted_consequence(key, 
                                                 dom, 
                                                 reference_chord, 
                                                 possible_octaves, 
                                                 ref_melodic_theme, 
                                                 ref_melodic_rhythm, 
                                                 True) 
    dom_octave = dom_melodic_theme[-1][-1][-1]
    # Generating "last"
    (last_melodic_theme, 
     last_melodic_rhythm) = last_chord(first_melodic_theme, 
                                       first_melodic_rhythm, 
                                       first, 
                                       last, 
                                       key, 
                                       possible_octaves, 
                                       strong_beats, 
                                       reference_dur,
                                       dom_octave)    
    # Generating "cad64"
    if 'cad64' in final_cs:
        bars_that_last = lasting_bars(final_cs, 'cad64', 'dom')
        (cad_melodic_theme, 
         cad_melodic_rhythm) = new_subordinate(key, 
                                               cad, 
                                               possible_octaves, 
                                               time_sig, 
                                               dom,
                                               dom_melodic_theme, 
                                               dom_melodic_rhythm, 
                                               True, 
                                               bars_that_last) 
    else:
        cad_melodic_theme = None
        cad_melodic_rhythm = None    
        
    # Generating "subdominant"
    if 'sub' in final_cs:
        if 'other' in final_pb:
            reference_is_first = np.random.choice([True,False])
            if not reference_is_first:
                reference_chord = other
                ref_melodic_theme = other_melodic_theme
                ref_melodic_rhythm = other_melodic_rhythm
        elif 'rep' in final_pb:
            reference_is_first = np.random.choice([True,False])
            if not reference_is_first:
                reference_chord = rep
                ref_melodic_theme = rep_melodic_theme
                ref_melodic_rhythm = rep_melodic_rhythm        
        else:
            reference_is_first = True
        if reference_is_first:
            reference_chord = first
            ref_melodic_theme = first_melodic_theme
            ref_melodic_rhythm = first_melodic_rhythm
            
        (sub_melodic_theme, 
         sub_melodic_rhythm) = melodic_transposition(subdom, 
                                                     reference_chord, 
                                                     key, 
                                                     possible_octaves, 
                                                     ref_melodic_theme, 
                                                     ref_melodic_rhythm)            
        
    else:
        sub_melodic_theme = None
        sub_melodic_rhythm = None
        
    # Generating "sub_subdoms"
    real_chords = [chord_group 
                   for chord_group in final_cs 
                   if type(chord_group) != float]
    if any('*' in chord_group 
           for chord_group in real_chords):
        
        subordinate = sub_subdoms[-1]           
        # random transformation
        transformation = np.random.choice(['inv', 
                                           'retro', 
                                           'retro-inv'])

        if transformation == 'inv':
            trans_melody, trans_rhythm = melodic_inversion(subordinate,
                                                           subdom, 
                                                           key, 
                                                           possible_octaves, 
                                                           sub_melodic_theme, 
                                                           sub_melodic_rhythm)
        elif transformation == 'retro':
            trans_melody, trans_rhythm = melodic_retrogression(subordinate,
                                                               subdom, 
                                                               key, 
                                                               possible_octaves, 
                                                               sub_melodic_theme, 
                                                               sub_melodic_rhythm)
        elif transformation == 'retro-inv':
            retro_melody, retro_rhythm = melodic_retrogression(subordinate,
                                                               subdom, 
                                                               key, 
                                                               possible_octaves, 
                                                               sub_melodic_theme, 
                                                               sub_melodic_rhythm)           
            trans_melody, trans_rhythm = melodic_inversion(subordinate,
                                                           subordinate, 
                                                           key, 
                                                           possible_octaves, 
                                                           retro_melody, 
                                                           retro_rhythm)                               
        sub_1_melodic_theme = trans_melody
        sub_1_melodic_rhythm = trans_rhythm
        sub_1 = subordinate
        
        subordinate_themes = []
        subordinate_rhythms = []
        
        for num_subs in range(len(sub_subdoms)-1):
            (subord_melodic_theme, 
             subord_melodic_rhythm) = melodic_transposition(sub_subdoms[num_subs], 
                                                         sub_subdoms[-1], 
                                                         key, 
                                                         possible_octaves, 
                                                         sub_1_melodic_theme, 
                                                         sub_1_melodic_rhythm)     
            subordinate_themes.append(subord_melodic_theme)
            subordinate_rhythms.append(subord_melodic_rhythm)
    
    else:
        subordinate_themes = None
        sub_3_melodic_theme = None
        sub_3_melodic_rhythm = None
        sub_2_melodic_theme = None
        sub_2_melodic_rhythm = None
        sub_1_melodic_theme = None
        sub_1_melodic_rhythm = None
        
    if subordinate_themes:
        if len(subordinate_themes) == 2:
            sub_3_melodic_theme = subordinate_themes[0]
            sub_3_melodic_rhythm = subordinate_rhythms[0]
            sub_3 = sub_subdoms[0]
            sub_2_melodic_theme = subordinate_themes[1]
            sub_2_melodic_rhythm = subordinate_rhythms[1]
            sub_2 = sub_subdoms[1]          
        elif len(subordinate_themes) == 1:
            sub_3_melodic_theme = None
            sub_3_melodic_rhythm = None
            sub_2_melodic_theme = subordinate_themes[0]
            sub_2_melodic_rhythm = subordinate_rhythms[0]
            sub_2 = sub_subdoms[0]
            
    # Optimization
    last_melodic_theme = theme_optimization(last_melodic_theme, 
                                             key, 
                                             last, 
                                             possible_octaves)
    dom_melodic_theme = theme_optimization(dom_melodic_theme, 
                                            key, 
                                            dom, 
                                            possible_octaves)
    if sub_melodic_theme:
        sub_melodic_theme = theme_optimization(sub_melodic_theme, 
                                               key, 
                                               subdom, 
                                               possible_octaves)
    if cad_melodic_theme:
        cad_melodic_theme = theme_optimization(cad_melodic_theme, 
                                                key, 
                                                cad, 
                                                possible_octaves)

    if sub_1_melodic_theme:
        sub_1_melodic_theme = theme_optimization(sub_1_melodic_theme, 
                                                key, 
                                                sub_1, 
                                                possible_octaves)      
    if sub_2_melodic_theme:
        sub_2_melodic_theme = theme_optimization(sub_2_melodic_theme, 
                                                key, 
                                                sub_2, 
                                                possible_octaves)     
    if sub_3_melodic_theme:
        sub_3_melodic_theme = theme_optimization(sub_3_melodic_theme, 
                                                key, 
                                                sub_3, 
                                                possible_octaves)             
    return (last_melodic_theme, 
            last_melodic_rhythm, 
            dom_melodic_theme, 
            dom_melodic_rhythm, 
            sub_melodic_theme, 
            sub_melodic_rhythm, 
            cad_melodic_theme, 
            cad_melodic_rhythm, 
            sub_1_melodic_theme, 
            sub_1_melodic_rhythm, 
            sub_2_melodic_theme, 
            sub_2_melodic_rhythm, 
            sub_3_melodic_theme, 
            sub_3_melodic_rhythm)

def final_melody_cs(final_cs, 
                    sub_3_melodic_theme, 
                    sub_2_melodic_theme, 
                    sub_1_melodic_theme, 
                    sub_melodic_theme, 
                    cad_melodic_theme, 
                    dom_melodic_theme, 
                    last_melodic_theme,
                    sub_3_melodic_rhythm, 
                    sub_2_melodic_rhythm, 
                    sub_1_melodic_rhythm, 
                    sub_melodic_rhythm, 
                    cad_melodic_rhythm, 
                    dom_melodic_rhythm, 
                    last_melodic_rhythm):
    
    possible_chords = ['sub***', 'sub**', 'sub*', 
                       'sub', 'cad64', 'dom', 'last']
    
    final_melody = []
    final_rhythm = []
    for chord in final_cs:
        if type(chord) != float:
            if chord == possible_chords[0]:
                for spans in sub_3_melodic_theme:
                    final_melody.append(spans)
                for spans in sub_3_melodic_rhythm:
                    final_rhythm.append(spans)                    
            elif chord == possible_chords[1]:
                for spans in sub_2_melodic_theme:
                    final_melody.append(spans)
                for spans in sub_2_melodic_rhythm:
                    final_rhythm.append(spans)                    
            elif chord == possible_chords[2]:
                for spans in sub_1_melodic_theme:
                    final_melody.append(spans)
                for spans in sub_1_melodic_rhythm:
                    final_rhythm.append(spans)                    
            elif chord == possible_chords[3]:
                for spans in sub_melodic_theme:
                    final_melody.append(spans)
                for spans in sub_melodic_rhythm:
                    final_rhythm.append(spans)                    
            elif chord == possible_chords[4]:
                for spans in cad_melodic_theme:
                    final_melody.append(spans)
                for spans in cad_melodic_rhythm:
                    final_rhythm.append(spans)                    
            elif chord == possible_chords[5]:
                for spans in dom_melodic_theme:
                    final_melody.append(spans)
                for spans in dom_melodic_rhythm:
                    final_rhythm.append(spans)                    
            elif chord == possible_chords[6]:
                for spans in last_melodic_theme:
                    final_melody.append(spans)
                for spans in last_melodic_rhythm:
                    final_rhythm.append(spans)
                    
    return final_melody, final_rhythm
