

import math
from util_functions.harmonic_morphing import roman_to_key, key_to_roman, new_subordinate

def data_initialisation(previous_tension,
                        current_tension, 
                        max_tension, 
                        chord_idx,
                        new_tension,
                        themes_data,
                        current_key):
        
    tension_decrease = previous_tension - current_tension
    bars_to_reach_theme = math.ceil((previous_tension - max_tension)/tension_decrease)
    if bars_to_reach_theme == 0:
        bars_to_reach_theme = 1
    reached_tension = current_tension - bars_to_reach_theme*tension_decrease
    if reached_tension < 0:
        reached_tension = 0

    dominating_idx = (chord_idx+bars_to_reach_theme)%len(new_tension)
    dominating_label = themes_data['chord_labels'][dominating_idx]   
    dominating_alphabet = roman_to_key(current_key, dominating_label.split('/')[-1])
    dominating_key = key_to_roman('C', dominating_alphabet)
    dominating_chord = dominating_label.split('/')[0]+'/'+dominating_key
    reference_melody = themes_data['melody_by_bars'][chord_idx-1]
    reference_rhythm = themes_data['rhythm_by_bars'][chord_idx-1]
    
    return (bars_to_reach_theme,
            reached_tension,
            tension_decrease,
            dominating_label,
            dominating_alphabet,
            dominating_key,
            dominating_chord,
            reference_melody,
            reference_rhythm)

def chord_generation(dominating_chord, 
                     themes_data,
                     chord_idx,
                     bars_to_reach_theme,
                     reached_tension, 
                     tension_decrease,
                     rows,
                     columns,
                     label_matrix,
                     distance_matrix,
                     increasing):
    
    previous_chord = dominating_chord      
    chord_sequence = themes_data['chord_labels'][:chord_idx]

    for new_bar_idx in range(bars_to_reach_theme):        
        predicted_tension = reached_tension - new_bar_idx*tension_decrease
        next_chord = new_subordinate(previous_chord, 
                                     dominating_chord, 
                                     rows,
                                     columns,
                                     label_matrix,
                                     distance_matrix,
                                     increasing, 
                                     predicted_tension, 
                                     chord_sequence)
        previous_chord = next_chord
        
    return next_chord