

import numpy as np
from AuToTen.setup import distance
from util_parameters.harmonic_param import (maj_diat_circ, 
                                            min_diat_circ, 
                                            sec_dom_fun, 
                                            maj_subdoms, 
                                            min_subdoms)

def key_quality(key):
    
    if key[0].isupper():
        key_roman = 'I'
    else:
        key_roman = 'i'
        
    return key_roman

def first_chord(key_roman):
    return key_roman + '/' + key_roman

def last_chord(key_roman):
    return key_roman + '/' + key_roman
    
def norm_chord(key_roman):
    
    probs = [1.0/(idx+1) for idx in range(4)]
    probs = [p/sum(probs) for p in probs]
  
    if key_roman.isupper():
        circle = maj_diat_circ[:4]
    else:
        circle = min_diat_circ[:4]
        
    norm = (np.random.choice(circle, p=probs) 
            + '/' + key_roman)
        
    return norm


def rep_chord(key_roman,
              norm, 
              first):
    
    if key_roman.isupper():
        possible_rep = maj_diat_circ[:4]
    else:
        possible_rep = min_diat_circ[:4]
        
    probs = [.5, .0, .0, .5]
    probs_rep = [probs[x] for x in range(len(possible_rep))
                 if possible_rep[x] != norm.split('/')[0]]
    probs_rep = [x/sum(probs_rep) for x in probs_rep]
    possible_rep = [x for x in possible_rep 
                    if x != norm.split('/')[0]]
    

    rep = (np.random.choice(possible_rep, p=probs_rep) 
           + '/' + key_roman)
    
    return rep

def subord_circle(right_branched, 
                  key_roman, 
                  norm):
    
    if right_branched:
        if key_roman.isupper():
            norm_diat_idx = maj_diat_circ.index(norm.split('/')[0])
            circle = (maj_diat_circ[norm_diat_idx:] 
                      + maj_diat_circ[:norm_diat_idx])
        else:
            norm_diat_idx = min_diat_circ.index(norm.split('/')[0])
            circle = (min_diat_circ[norm_diat_idx:] 
                      + min_diat_circ[:norm_diat_idx])     
    else:
        if key_roman.isupper():
            des_circ = [x for x 
                        in maj_diat_circ[::-1]]
        else:
            des_circ = [x for x 
                        in min_diat_circ[::-1]]

        norm_diat_idx = des_circ.index(norm.split('/')[0])
        circle = (des_circ[norm_diat_idx:] 
                  + des_circ[:norm_diat_idx])
    circle = circle[1:4]
    circle = [x+'/'+key_roman for x in circle]
    
    # include as possibilities the secondary dominants of norm
    norm_secondary_tonic = norm.split('/')[0].split('-')[0].split('+')[0].split('7')[0]
    for x in sec_dom_fun:
        circle.append(x+'/'+norm_secondary_tonic)
        
    return circle


def subord_chord(right_branched, 
                 key_roman, 
                 dominating_chord, 
                 first, 
                 key, 
                 rep, 
                 norm):
    
    circle = subord_circle(right_branched, 
                           key_roman, 
                           dominating_chord)
       
    # accept as possible subordinates those that would leave
    # a stable prolongational structure
    if rep:
        possible_subordinate = [x for x in circle
                                if (distance(key, dominating_chord, x)[0] 
                                    <= distance(key, first, x)[0]) 
                                and (distance(key, dominating_chord, x)[0] 
                                     <= distance(key, rep, x)[0])
                                and (distance(key, dominating_chord, x)[0] 
                                     <= distance(key, norm, x)[0])]
    else:
        possible_subordinate = [x for x in circle
                                if (distance(key, dominating_chord, x)[0] 
                                    <= distance(key, first, x)[0])
                                and (distance(key, dominating_chord, x)[0] 
                                     <= distance(key, norm, x)[0])]        
    
    if len(possible_subordinate) == 0: 
        subordinate = dominating_chord
    else:
        subordinate = np.random.choice(possible_subordinate)
    
    return subordinate
                    
def dom_chord(key_roman):
    return (np.random.choice(['V', 'V7']) 
           + '/' + key_roman)
                    
def cad_chord(first):
    return first

def possible_subs(key_roman, 
                  dominating_chord, 
                  final_cs):
    
    if key_roman.isupper():
        subds = [x+'/'+key_roman 
                 for x in maj_subdoms]
    else:
        subds = [x+'/'+key_roman 
                 for x in min_subdoms]
    if 'cad64' in final_cs:
        subs = subds
    else:
        doms = [x+'/'+dominating_chord
                for x in sec_dom_fun]
        subs = subds + doms
    return subs

def subdom_chord(key_roman, key,
                 first, 
                 dom, 
                 final_cs):
        
    subs = possible_subs(key_roman, 
                         'V', 
                         final_cs)
    
    # accept as possible subdominants those that would leave
    # a stable prolongational structure
    possible = [chord for chord in subs
               if distance(key, dom, chord)[0] 
                <= distance(key, first, chord)[0]]
    subdom = np.random.choice(possible)
    return subdom

def sub_subdom_chords(key_roman, 
                      subdom, 
                      key, 
                      dom, 
                      final_cs):
    
    previous_subdom = str(subdom)
    subdoms_sequence = []
    for subordinate in range(3): # max number of subordinates
        dominating_chord = previous_subdom.split('/')[0].split('-')[0].split('7')[0]
        dom_key = previous_subdom.split('/')[-1]
        
        if dom_key == key_roman:
            subs = possible_subs(key_roman, 
                                 dominating_chord, 
                                 final_cs)     
        else:
            subs = possible_subs(key_roman, 
                                 dom_key, 
                                 final_cs)                
        
        # accept as possible sub_subdominants those that would leave
        # a stable prolongational structure
        possible = [chord for chord in subs
                   if distance(key, previous_subdom, chord)[0] 
                    < distance(key, dom, chord)[0] 
                    and chord not in subdoms_sequence] 
        
        if len(possible) == 0 and key_roman.isupper():
            previous_subdom = 'IV/I'
        elif len(possible) == 0 and key_roman.islower():
            previous_subdom = 'iv/i'
        else:
            previous_subdom = np.random.choice(possible)
        subdoms_sequence.append(previous_subdom)
    return subdoms_sequence
    
    
    
    
    
    
    
    
    
    
    
    
    
