

from util_functions import harmonic_components
from util_parameters.harmonic_param import (maj_diat_circ, 
                                            min_diat_circ, 
                                            sec_dom_fun, 
                                            maj_subdoms, 
                                            min_subdoms)
def pb_chords(key, final_pb):
    
    # Calculating the label of the input key
    key_roman = harmonic_components.key_quality(key)

    # Forcing 'first' chord to be the tonic chord
    first = harmonic_components.first_chord(key_roman)

    # Probabilistic generation of 'norm' chord
    norm = harmonic_components.norm_chord(key_roman)

    if 'rep' in final_pb:
        # Stochastic generation of 'rep' chord
        rep = harmonic_components.rep_chord(key_roman, 
                                            norm, 
                                            first)
    else:
        rep = None

    if 'other' in final_pb:
        # Stochastic generation of 'other' chord
        other = harmonic_components.rep_chord(key_roman, 
                                              norm, 
                                              first)    
    else:
        other = None
        
    subordinates = {}

    if '*norm' in final_pb:
        subordinates['*norm'] = [False, 
                                 norm] # i.e left-branched
    if 'norm*' in final_pb:
        subordinates['norm*'] = [True, 
                                 norm] # i.e right-branched
    if '*other' in final_pb:
        subordinates['*other'] = [False, 
                                 other] # i.e left-branched
    if 'other*' in final_pb:
        subordinates['other*'] = [True, 
                                  other] # i.e right-branched
        
    left_subnorm = None
    right_subnorm = None            
    left_subother = None          
    right_subother = None
    for subord in subordinates:
        # Stochastic generation of '*' chords
        subnorm = (harmonic_components
                   .subord_chord(subordinates[subord][0], 
		                  key_roman, 
		                  subordinates[subord][1], 
		                  first, 
		                  key, 
		                  rep, 
                                  norm))

        if subord == '*norm':
            left_subnorm = subnorm
        elif subord == 'norm*':
            right_subnorm = subnorm
        elif subord == '*other':
            left_subother = subnorm
        elif subord == 'other*':
            right_subother = subnorm
       
    return (key_roman, 
            first, 
            norm, 
            rep, 
            other, 
            left_subnorm, 
            right_subnorm, 
            left_subother, 
            right_subother)
            
def cs_chords(key_roman, 
              key, 
              first, 
              final_cs):        
    
    # Deterministic generation of 'last' chord
    last = harmonic_components.last_chord(key_roman)
    
    # Stochastic generation of 'dom' chord
    dom = harmonic_components.dom_chord(key_roman)
    
    # Deterministic generation of 'cad64' chord
    cad = harmonic_components.cad_chord(first)
    
    # Stochastic generatino of 'sub' chord
    subdom = harmonic_components.subdom_chord(key_roman, 
                                              key, 
                                              first, 
                                              dom, 
                                              final_cs)

    # Stochastic generation of 'sub*(**)' chords
    sub_subdoms = (harmonic_components
                   .sub_subdom_chords(key_roman, 
                                      subdom, 
                                      key, 
                                      dom, 
                                      final_cs))
    return (last, 
            dom, 
            cad, 
            subdom, 
            sub_subdoms[::-1])
