

from AuToTen.setup import common, distance, classes


def new_subordinate(previous_chord, 
                    dominating_chord, 
                    rows,
                    columns,
                    label_matrix,
                    distance_matrix,
                    increasing, 
                    current_tps_tension, 
                    chord_seq):

    previous_idx = rows.index(previous_chord)
    distance_transition_vector = distance_matrix[previous_idx]
    dist_diff = [abs(current_tps_tension-t) 
                 for t in distance_transition_vector]
    possible_idx = [idx for idx in range(len(dist_diff))
                    if dist_diff[idx] == min(dist_diff)]
    
    # Avoid repetitions, if possible
    new_possible_idx = [idx for idx in possible_idx
                        if rows[idx] not in chord_seq]
    if len(new_possible_idx) != 0:
        possible_idx = new_possible_idx
        
    if increasing:    # Prioritise minors and diminished, when possible   
        preferred_idx = [idx for idx in possible_idx 
                         if '7' in label_matrix[previous_idx][idx]
                         or label_matrix[previous_idx][idx].split('/')[0].split('-')[0][-1].islower()]
        if len(preferred_idx) == 0:
            preferred_idx = possible_idx
    else:
        preferred_idx = possible_idx
    
    dom_dist = [distance('C', dominating_chord, label_matrix[previous_idx][idx])[0] 
                for idx in preferred_idx]
    next_idx = preferred_idx[dom_dist.index(max(dom_dist))]
    next_chord = label_matrix[previous_idx][next_idx]

    return next_chord 

def key_to_roman(original_key, destination_key):
    
    # Calculating original scales
    if original_key[0].isupper():
        original_tonic = 'I/I'
    else:
        original_tonic = 'i/i'
    scale = classes.parser.Scale(original_key, 
                                 original_tonic).local_sorted_scale()
    roman_degrees = ('i', 'ii', 'iii', 'iv', 'v', 'vi', 'vii')        
    enh = common.enharmonics
    
    # root in diatonic scale
    if destination_key.lower() in scale:
        if destination_key[0].isupper():
            destination_roman = roman_degrees[scale.index(destination_key.lower())].upper()
        else:
            destination_roman = roman_degrees[scale.index(destination_key.lower())]
    elif destination_key.lower()+'b' in scale:
        if destination_key[0].isupper():
            destination_roman = '#' + roman_degrees[scale.index(destination_key.lower()+'b')].upper()
        else:
            destination_roman = '#' + roman_degrees[scale.index(destination_key.lower()+'b')]        
    elif destination_key.lower()+'#' in scale:
        if destination_key[0].isupper():
            destination_roman = 'b' + roman_degrees[scale.index(destination_key.lower()+'#')].upper()
        else:
            destination_roman = 'b' + roman_degrees[scale.index(destination_key.lower()+'#')]        
    elif destination_key.lower()[:-1] in scale:
        if destination_key[0].isupper():
            destination_roman = destination_key[-1] + roman_degrees[scale.index(destination_key.lower()[:-1])].upper()
        else:
            destination_roman = destination_key[-1] + roman_degrees[scale.index(destination_key.lower()[:-1])]   
    elif (destination_key.lower() in enh 
          and enh[destination_key.lower()] in scale):
        if destination_key[0].isupper():
            destination_roman = roman_degrees[scale.index(enh[destination_key.lower()])].upper()
        else:
            destination_roman = roman_degrees[scale.index(enh[destination_key.lower()])] 
            
    return destination_roman

def roman_to_key(original_key, current_roman):
    
    if original_key[0].isupper():
        original_tonic = 'I/I'
    else:
        original_tonic = 'i/i'    
    sorted_scale = classes.parser.Scale(original_key, 
                                        original_tonic).local_sorted_scale()
    roman_degrees = ('i', 'ii', 'iii', 'iv', 'v', 'vi', 'vii')     
    degree_idx = roman_degrees.index(current_roman.split('b')[-1].split('#')[-1].lower())

    if 'b' in current_roman:
        if sorted_scale[degree_idx][-1] == '#':
            current_key = sorted_scale[degree_idx][:-1]
        else:
            current_key = sorted_scale[degree_idx]+'b'
    elif '#' in current_roman:
        if sorted_scale[degree_idx][-1] == 'b':
            current_key = sorted_scale[degree_idx][:-1]
        else:
            current_key = sorted_scale[degree_idx]+'#'            
    else:
        current_key = sorted_scale[degree_idx]
        
    if current_roman[-1].isupper():
        current_key = current_key[0].upper()+current_key[1:]

    right_keys = common.sharp_keys[:8]
    flat_right_keys = [item for sublist in right_keys for item in sublist]      
    left_keys = common.flat_keys[1:9]
    flat_left_keys = [item for sublist in left_keys for item in sublist]  
    all_keys = flat_right_keys + flat_left_keys
    enh = common.enharmonics    
    if current_key not in all_keys:
        current_key = enh[current_key.lower()]
        if current_roman[-1].isupper():
            current_key = current_key[0].upper()+current_key[1:]
        
    return current_key



