

import pandas as pd
from AuToTen.setup import classes, common

def chord_data(key, 
               current_chord, 
               possible_octaves):
    
    # Calculating chordal notes
    chordal_notes = [classes.notes.Notes(key, 
                                         current_chord).root(), 
                     classes.notes.Notes(key, 
                                         current_chord).third(),
                     classes.notes.Notes(key, 
                                         current_chord).fifth(),
                     classes.notes.Notes(key, 
                                         current_chord).seventh()]
    
    # Update chordal notes if there is no 7th
    if not chordal_notes[-1]:
        chordal_notes = chordal_notes[:-1]
    
    # Calculating diatonic scale
    scale = classes.parser.Scale(key, 
                                 current_chord).local_diatonic_scale()
    # Adapting scale to chordal notes 
    if chordal_notes[0] not in scale:      # Root
        idx = None
        for x in range(len(scale)):
            if scale[x][0] == chordal_notes[0][0]:
                idx = x
        scale[idx] = chordal_notes[0]
    if chordal_notes[1] not in scale:    # Third
        idx = None
        for x in range(len(scale)):
            if scale[x][0] == chordal_notes[1][0]:
                idx = x
        scale[idx] = chordal_notes[1]
    if chordal_notes[2] not in scale:
        idx = None
        for x in range(len(scale)):        # Fifth
            if scale[x][0] == chordal_notes[2][0]:
                idx = x
        scale[idx] = chordal_notes[2]

    # Defining the melodic range
    melodic_range = [note+str(octave) 
                     for octave in possible_octaves 
                     for note in scale]
            
    return chordal_notes, scale, melodic_range


def time_sig_data(time_sig):
    
    division, duration_ref = time_sig.split('/')
    
    if int(division) in (2,6):
        num_beats = 2
        strong_beats = (True, 
                        False)
    elif int(division) in (3,9):
        num_beats = 3
        strong_beats = (True, 
                        False, 
                        False)
    elif int(division) in (4,12):
        num_beats = 4
        strong_beats = (True, 
                        False, 
                        True, 
                        False)
    
    if int(duration_ref)== 4:
        subdivision = 2
        reference_dur = 1.0
    else:
        subdivision = 3
        reference_dur = 1.5
  
    return num_beats, subdivision, strong_beats, reference_dur


def bar_notes_range(num_beats, 
                    subdivision, 
                    character):
    
    # Defining melodic rhythm range
    # with regards to character
    note_ranges = {'character': ['adagio', 
                                 'andante', 
                                 'allegro', 
                                 'presto'],
            'min notes/bar': [1, 1, 
                              num_beats, 
                              subdivision*num_beats],
            'max notes/bar': [num_beats, 
                              subdivision*num_beats, 
                              2*subdivision*num_beats, 
                              2*subdivision*num_beats]}

    df = pd.DataFrame(note_ranges, 
                      columns = ['character',
                                 'min notes/bar', 
                                 'max notes/bar'])
    
    note_range_idx = note_ranges['character'].index(character)
    note_range = (note_ranges['min notes/bar'][note_range_idx],
              note_ranges['max notes/bar'][note_range_idx])

    return df, note_range