

from util_parameters.melodic_param import possible_octaves
from util_functions.melodic_transformations import (melodic_inversion,
                                                    melodic_transposition, 
                                                    melodic_retrogression)

def mel_development(transformations, 
                    melody, 
                    rhythm, 
                    chord_labels, 
                    left_chords, 
                    right_chords, 
                    current_subordinates, 
                    key, 
                    chord_seq):
    
    new_melodies = []
    new_rhythms = []
    for transformation in transformations:
        new_melody = []
        new_rhythm = []
        span = 0
        for idx in range(len(melody)):

            # Reference data
            reference_chord = chord_labels[span]
            ref_melodic_theme = [melody[idx]]
            ref_melodic_rhythm = [rhythm[idx]]

            # Find rests
            if chord_seq[span] in left_chords:
                left_rest = True
                right_rest = False
                rest_dur = rhythm[idx]
            elif chord_seq[span] in right_chords:
                left_rest = False
                right_rest = True
                rest_dur = rhythm[idx+1]
            else:
                left_rest = False
                right_rest = False

            if melody[idx] != ['rest']:

                # Current data
                current_chord = current_subordinates[span]

                # Apply current transformation
                if transformation == 'inv':

                    trans_melody, trans_rhythm = melodic_inversion(current_chord, 
                                                                   reference_chord, 
                                                                   key, 
                                                                   possible_octaves, 
                                                                   ref_melodic_theme, 
                                                                   ref_melodic_rhythm)

                elif transformation == 'retro':

                    trans_melody, trans_rhythm = melodic_retrogression(current_chord, 
                                                                       reference_chord, 
                                                                       key, 
                                                                       possible_octaves, 
                                                                       ref_melodic_theme, 
                                                                       ref_melodic_rhythm)

                elif transformation == 'retro-inv':

                    retro_melody, retro_rhythm = melodic_retrogression(current_chord, 
                                                                       reference_chord, 
                                                                       key, 
                                                                       possible_octaves, 
                                                                       ref_melodic_theme, 
                                                                       ref_melodic_rhythm) 

                    trans_melody, trans_rhythm = melodic_inversion(current_chord, 
                                                                   current_chord, 
                                                                   key, 
                                                                   possible_octaves, 
                                                                   retro_melody, 
                                                                   retro_rhythm)      

            if left_rest and melody[idx-1] == ['rest']:
                new_melody.append([['rest']]+trans_melody)
                new_rhythm.append(rest_dur+trans_rhythm)
                span += 1
            elif right_rest and melody[idx+1] == ['rest']:
                new_melody.append(trans_melody+[['rest']])
                new_rhythm.append(trans_rhythm+rest_dur)
            elif right_rest:
                span += 1
            elif not left_rest and not right_rest:
                new_melody.append(trans_melody)
                new_rhythm.append(trans_rhythm)
                span += 1

        new_melodies.append(new_melody)
        new_rhythms.append(new_rhythm)

    return new_melodies, new_rhythms