

import numpy as np
from collections import Counter
from AuToTen.setup import classes

def first_skeleton_melody(possible_octaves, 
                          current_chordal_notes, 
                          strong_beats, 
                          current_melodic_range, 
                          final_pb, 
                          previous_octave):
    
    # Generating first note
    # (it will always be the chord's root)
    is_not_highest = False    
    while not is_not_highest:
        if previous_octave:
            first_note = (current_chordal_notes[0]
                          +previous_octave)   
            while first_note == current_melodic_range[-1]:
                first_note = (current_chordal_notes[0]
                              +str(np.random.choice(possible_octaves)))                
        else:
        # to avoid having a head being the highest note
            '''
            first_note = (current_chordal_notes[0]
                          +str(np.random.choice(possible_octaves)))
            '''
            first_note = (current_chordal_notes[0]
                          +str(possible_octaves[0]))            
        if first_note != current_melodic_range[-1]:
            is_not_highest = True

    # Random generation of last note
    last_note = np.random.choice(current_chordal_notes)

    # Calculating the number of bars "first" lasts
    bars_in_group = 1
    bar_is_nan = True
    bar_counter = 0
    while bar_is_nan:
        bar = final_pb[1:][bar_counter]
        if type(bar)==float:
            bars_in_group += 1
        else:
            bar_is_nan = False
        bar_counter += 1   
        
    # Calculating intermediate notes
    num_strong_beats = [beat 
                        for beat in strong_beats 
                        if beat 
                        for bar in range(bars_in_group)]
    
    intermediate_notes = []
    for strong_beat in range(len(num_strong_beats)-1):
        interm_note_idx = np.random.choice([idx 
                                            for idx 
                                            in range(len(current_chordal_notes))])
        intermediate_notes.append(current_chordal_notes[interm_note_idx])
    intermediate_notes.append(last_note)
    
    # Statistically defining octaves
    previous_skel_note = first_note
    intermediate_melody = []
    for note in intermediate_notes:
        possible_notes = [note+str(octv) 
                          for octv in possible_octaves]
        diatonic_distances = [1
                              +abs(current_melodic_range.index(previous_skel_note)
                               - current_melodic_range.index(possible_note))
                             for possible_note in possible_notes]
        probs = [1.0/dist for dist in diatonic_distances]
        probs_n = [prob/sum(probs) for prob in probs]
        octave_idx = np.random.choice([poss 
                                       for poss in range(len(possible_notes))], 
                                      p=probs_n)
        previous_skel_note = possible_notes[octave_idx]
        intermediate_melody.append(previous_skel_note)
        
    return (first_note, 
            intermediate_melody[-1], 
            intermediate_melody[:-1])

def span_melody_generation(original_note, 
                           destination_note, 
                           current_chordal_notes, 
                           current_melodic_range, 
                           note_range, 
                           strong_beats):
    
    strong_counter = Counter(strong_beats)[True]    
    if strong_counter == 1:
        beats_in_span = len(strong_beats)
    else:
        beats_in_span = 2
        
    min_notes_bar, max_notes_bar = note_range
    min_notes_span = min_notes_bar//beats_in_span
    max_notes_span = max_notes_bar//beats_in_span
    
    previous_note = original_note
    previous_note_chordal = True
    
    possible_melody = [original_note]
    should_generate = True
    allowed_interval = 7   # an octave
    while should_generate:
        # Probabilities inveresely related to distance to original_note
        next_note_probs_original = [1.0/(1.0
                                         +abs(note_idx
                                                 -current_melodic_range.index(original_note))) 
                           if abs(note_idx
                                  -current_melodic_range.index(previous_note)) 
                           <= allowed_interval else 0.0 
                           for note_idx in range(len(current_melodic_range))]
        next_note_probs_destination = [1.0/(1.0
                                            +abs(note_idx
                                                 -current_melodic_range.index(destination_note))) 
                           if abs(note_idx
                                  -current_melodic_range.index(previous_note)) 
                           <= allowed_interval else 0.0 
                           for note_idx in range(len(current_melodic_range))]
        next_note_probs = [next_note_probs_original[prob_idx]
                           +next_note_probs_destination[prob_idx] 
                           for prob_idx in range(len(next_note_probs_original))]
        if not previous_note_chordal:    # next_note should follow ornamental rules  
            next_note_probs = [next_note_probs[prob_idx] 
                               if abs(prob_idx
                                      -current_melodic_range.index(previous_note)) 
                               <= 1 
                               else 0.0 
                               for prob_idx in range(len(current_melodic_range))]
        #else: next note could be whatever

        next_note_probs_n = [prob/sum(next_note_probs) for prob in next_note_probs]
        next_note = np.random.choice(current_melodic_range, p=next_note_probs_n)
        
        if (abs(current_melodic_range.index(next_note) 
                - current_melodic_range.index(previous_note)) 
            >= 2):    # if there is jump greater than a third
            allowed_interval = 4    # a fifth
        else:
            allowed_interval = 7
           
        if next_note in current_chordal_notes:
            previous_note_chordal = True
        else:
            previous_note_chordal = False
        
        possible_melody.append(next_note)
        previous_note = next_note  
        
        if len(possible_melody) == max_notes_span:
            intervals = [abs(current_melodic_range.index(possible_melody[note_idx])
                            - current_melodic_range.index(possible_melody[note_idx+1])) 
                         for note_idx in range(len(possible_melody)-1)]
            if any(interval 
                   > 7 for interval in intervals):
                # initial conditions
                possible_melody = [original_note]
                allowed_interval = 7   # an octave
            else:
                should_generate = False   
        elif len(possible_melody) > max_notes_span:
            if max_notes_span == 1:
                possible_melody = [original_note]
                should_generate = False
            else:
                # initial conditions
                possible_melody = [original_note]
                allowed_interval = 7   # an octave
        
    return possible_melody, beats_in_span

def distance_to_destination(possible_melody, 
                            current_melodic_range, 
                            destination_note):
    
    return [abs(current_melodic_range.index(note)
                -current_melodic_range.index(destination_note)) 
            for note in possible_melody]


def fit_destination_note(possible_melody, 
                         dis, 
                         note_range, 
                         last_note, 
                         destination_is_last):
    
    # selecting the most appropriate length for a melody 
    # making sure it is within the melody's character range
    if destination_is_last:
        if last_note in possible_melody:
            last_note_idx = possible_melody.index(last_note)
            return possible_melody[:last_note_idx+1]          
        else:
            min_dist_idx = [note_idx 
                            for note_idx in range(len(dis)) 
                            if dis[note_idx] == min(dis) 
                            and len(possible_melody) >= note_range[0] 
                            and len(possible_melody) <= note_range[1]]  
            if len(min_dist_idx) == 0:
                min_dist_idx = [note_idx 
                                for note_idx in range(len(dis))]
            last_note_idx = np.random.choice(min_dist_idx)
            if last_note_idx == len(possible_melody)-1:
                return possible_melody[:-1]+[last_note]
            else:
                return possible_melody[:last_note_idx+1]+[last_note]          

    else:
        min_dist_idx = [note_idx 
                        for note_idx in range(len(dis)) 
                        if dis[note_idx] == min(dis) 
                        and len(possible_melody) >= note_range[0] 
                        and len(possible_melody) <= note_range[1]]
        if len(min_dist_idx) == 0:
            min_dist_idx = [note_idx 
                            for note_idx in range(len(dis))]
        last_note_idx = np.random.choice(min_dist_idx)
        return possible_melody[:last_note_idx+1]


def melody_weighting(key, 
                     original_chord,
                     span_melody, 
                     dis, 
                     current_melodic_range, 
                     previous_melody=False):
    
    # 1) note variation
    variation = len(Counter(span_melody).values())/len(span_melody)
    
    # 2) TPS' stability
    basic_space = classes.space.Space(key, 
                                      original_chord).basic_space()*3
    span_stability = [basic_space[current_melodic_range.index(note)]/max(basic_space) 
                       for note in span_melody]
    stability = sum(span_stability)/len(span_melody)
    
    # 3) intervalic jumps
    if max(dis) != 0:
        span_jumps = [jump/max(dis) for jump in dis]
    else:
        span_jumps = [jump for jump in dis]
    jumps = sum(span_jumps)/len(dis)
    
    # 4) similar note density
    similar_density = 0.0
    if previous_melody:
        if len(previous_melody) == len(span_melody):
            similar_density = 1.0
            
    return (variation, 
            stability, 
            jumps, 
            similar_density, 
            span_stability)
