

import math
from util_functions.melodic_components import chord_data

def new_melody(reference_rhythm, 
               max_profile, 
               max_tension, 
               reference_key, 
               next_chord, 
               possible_octaves, 
               reference_melody,
               increasing, 
               dominating_chord, 
               current_tension):    
    
    num_notes = len(reference_rhythm)
    octave_range = int((max_profile-max_tension)/3)
    
    if current_tension <= max_tension + octave_range:
        reference_octave = str(4)
    elif current_tension <= max_tension + octave_range*2:
        reference_octave = str(5)
    else:
        reference_octave = str(6)

    root_notes_idxs = {1:(0,), 2:(0,2), 3:(0,2,4), 4:(0,1,2,4), 
                      5:(0,1,2,3,4), 6:(0,1,2,3,4,5), 
                      7:(0,1,2,3,4,5,6), 8:(0,1,2,0,1,2,3,4),
                      9:(0,1,2,1,2,3,2,3,4), 10:(0,1,2,3,4,0,1,2,3,4),
                      11:(0,1,2,3,4,0,1,2,3,4,5),
                      12:(0,1,2,3,4,0,1,2,3,4,5,6),
                      13:(0,1,2,3,4,5,0,1,2,3,4,5,6),
                      14:(0,0,1,1,2,2,3,3,4,4,5,5,6,6),
                      15:(0,1,2,1,2,3,2,3,4,3,4,5,4,5,6),
                      16:(0,1,2,0,1,2,3,4,0,1,2,0,1,2,3,4)}
    
    third_notes_idxs = {1:(0,), 2:(0,2), 3:(0,2,5), 4:(0,1,2,5), 
                      5:(0,1,2,4,5), 6:(0,1,2,3,4,5), 
                      7:(0,1,2,3,4,5,6), 8:(0,1,2,0,1,2,4,5),
                      9:(0,1,2,1,2,3,2,3,5), 10:(0,1,2,3,0,1,2,3,4,5),
                      11:(0,1,2,3,4,0,1,2,3,4,5),
                      12:(0,1,2,3,4,0,1,2,3,4,5,6),
                      13:(0,1,2,3,4,5,0,1,2,3,4,5,6),
                      14:(0,0,1,1,2,2,3,3,4,4,5,5,6,6),
                      15:(0,1,2,1,2,3,2,3,4,3,4,5,4,5,6),
                      16:(0,1,2,0,1,2,3,4,0,1,2,0,1,2,3,5)} 
    
    fifth_notes_idxs = {1:(0,), 2:(0,3), 3:(0,3,5), 4:(0,2,3,5), 
                      5:(0,1,2,3,5), 6:(0,1,2,3,4,5), 
                      7:(0,1,2,3,4,5,6), 8:(0,1,2,0,1,2,3,5),
                      9:(0,1,2,1,2,3,2,3,5), 10:(0,1,2,3,0,1,2,3,4,5),
                      11:(0,1,2,3,4,0,1,2,3,4,5),
                      12:(0,1,2,3,4,0,1,2,3,4,5,6),
                      13:(0,1,2,3,4,5,0,1,2,3,4,5,6),
                      14:(0,0,1,1,2,2,3,3,4,4,5,5,6,6),
                      15:(0,1,2,1,2,3,2,3,4,3,4,5,4,5,6),
                      16:(0,1,2,0,1,2,3,4,0,1,2,0,1,2,3,5)}     
    
    
    (current_chordal_notes, 
     current_scale, 
     current_melodic_range) = chord_data(key=reference_key, 
                                         current_chord=next_chord, 
                                         possible_octaves=(4,5,6,7))    
    current_chordal_notes = current_chordal_notes[:3]
    
    (previous_chordal_notes, 
     previous_scale, 
     previous_melodic_range) = chord_data(key=reference_key, 
                                         current_chord=dominating_chord, 
                                         possible_octaves=(4,5,6,7))        
    
    previous_mel_note = [x 
                         for x in reference_melody 
                         if x != 'rest'][-1]
    if previous_mel_note not in previous_melodic_range:
        previous_mel_note = [x for x in previous_melodic_range
                             if x[0] == previous_mel_note[0]][0][:-1]+previous_mel_note[-1]

    previous_idx = previous_melodic_range.index(previous_mel_note)
    
    chordal_notes_in_range = [c 
                              for c in current_melodic_range 
                              if c[:-1] in current_chordal_notes]
    
    if increasing:
        diff = [current_melodic_range.index(c)-previous_idx
                if current_melodic_range.index(c) > previous_idx
                else math.inf
                for c in chordal_notes_in_range]
        new_first_note = chordal_notes_in_range[diff.index(min(diff))]
        if int(new_first_note[-1]) > int(reference_octave):
            new_first_note = chordal_notes_in_range[diff.index(min(diff))][:-1]+reference_octave
        first_note_idx = current_melodic_range.index(new_first_note)
    else:
        diff = [current_melodic_range.index(c)-previous_idx
                if current_melodic_range.index(c) < previous_idx
                else -1*math.inf
                for c in chordal_notes_in_range]
        new_first_note = chordal_notes_in_range[diff.index(max(diff))]
        if int(new_first_note[-1]) < int(reference_octave):        
            new_first_note = chordal_notes_in_range[diff.index(max(diff))][:-1]+reference_octave
        if new_first_note not in current_melodic_range:
            new_first_note = [x for x in current_melodic_range
                                 if x[0] == new_first_note[0]][0][:-1]+previous_mel_note[-1]
        first_note_idx = current_melodic_range.index(new_first_note)        
        
        
    if new_first_note[:-1] == current_chordal_notes[0]:
        new_notes_idxs = root_notes_idxs
    elif new_first_note[:-1] == current_chordal_notes[1]:
        new_notes_idxs = third_notes_idxs
    elif new_first_note[:-1] == current_chordal_notes[2]:
        new_notes_idxs = fifth_notes_idxs        
        
    subord_melody = [current_melodic_range[first_note_idx + idx] 
                     if first_note_idx + idx < len(current_melodic_range)
                     else current_melodic_range[first_note_idx + idx - 7]
                     for idx in new_notes_idxs[num_notes]]
    if not increasing:
        subord_melody = subord_melody[::-1]
        
    return subord_melody