

import itertools
import numpy as np
import math
from collections import Counter
from util_functions.melodic_components import time_sig_data

def rhythm_assignment(time_sig, 
                      melodic_theme, 
                      melodic_beats_in_span, 
                      melodic_stability):
    (num_beats, 
     subdivision, 
     strong_beats, 
     reference_dur) = time_sig_data(time_sig)
    
    flat_melodic_theme = [item 
                          for sublist in melodic_theme 
                          for item in sublist]
    flat_melodic_stability = [item 
                              for sublist in melodic_stability 
                              for item in sublist]
    durations_palette = [.25,.5,.75,1.,1.5,2.,3.,4.,4.5,6.]
    melodic_rhythm = []
    for span in range(len(melodic_theme)):
        # max duration of a note considering the rest of the notes are 16ths
        maximum_duration = (melodic_beats_in_span[0]*reference_dur
                            -(len(melodic_theme[span])-1)*.25)
        minimum_duration = melodic_beats_in_span[0]*reference_dur/len(melodic_theme[span])
        current_palette = [dur for dur in durations_palette 
                          if dur >= minimum_duration]
        possible_durations = [dur 
                              for dur in current_palette 
                              if dur <= maximum_duration]
        head_probs = [dur/sum(possible_durations) 
                      for dur in possible_durations]
        head_dur = np.random.choice(possible_durations, 
                                    p=head_probs)
        if len(melodic_theme[span]) == 1:
            head_dur = max(possible_durations)
        span_rhythm = [head_dur]
        for note in range(len(melodic_theme[span])-1):           
            maximum_duration = (melodic_beats_in_span[0]*reference_dur 
                                - sum(span_rhythm)
                                -(len(melodic_theme[span][note+1:])-1)*.25)
            provisional_palette = durations_palette+[0.0]
            possible_durations = [dur 
                                  for dur in durations_palette 
                                  if dur <= maximum_duration]            
            note_stability = melodic_stability[span][note+1]
            dur_weightings = [dur/max(possible_durations) 
                             for dur in possible_durations]
            diff = [1.0/abs(dur-note_stability) 
                    if abs(dur-note_stability) != 0.0 
                    else 1.0 
                   for dur in dur_weightings]
            metre_stability = [1.0 
                               if (sum(span_rhythm)+dur)%reference_dur == 0 
                               else 0.0 
                              for dur in possible_durations]
            probs = [diff[dur]*metre_stability[dur] 
                     for dur in range(len(possible_durations))]
            if sum(probs) == 0.0:
                probs = [diff[dur]+metre_stability[dur] 
                         for dur in range(len(possible_durations))]                
            probs = [p/sum(probs) for p in probs]
            span_rhythm.append(np.random.choice(possible_durations, 
                                                p=probs))
        melodic_rhythm.append(span_rhythm)
    # edit last span if necessary
    if sum(span_rhythm) != melodic_beats_in_span[-1]*reference_dur:
        melodic_rhythm[-1][-1] = (melodic_beats_in_span[-1]*reference_dur
                                  - sum(span_rhythm[:-1])) 
        
    x = [item for sublist in melodic_rhythm for item in sublist]
    if sum(x) != reference_dur*num_beats:
        print('ERROR')
    return melodic_rhythm
