

import numpy as np
from util_functions.melodic_components import (chord_data, 
                                               time_sig_data, 
                                               bar_notes_range)
from util_functions.melodic_generation import (first_skeleton_melody, 
                                               span_melody_generation, 
                                               distance_to_destination, 
                                               fit_destination_note, 
                                               melody_weighting)

def new_melodic_components(key, 
                           current_chord, 
                           possible_octaves, 
                           time_sig, 
                           character, 
                           final_pb, 
                           previous_octave):

    (current_chordal_notes, 
     current_scale, 
     current_melodic_range) = chord_data(key, 
                                         current_chord, 
                                         possible_octaves)

    (num_beats, 
     subdivision, 
     strong_beats, 
     reference_dur) = time_sig_data(time_sig)

    (df, 
     note_range) = bar_notes_range(num_beats, 
                                   subdivision, 
                                   character)

    (first_note, 
     last_note, 
     intermediate_melody) = first_skeleton_melody(possible_octaves, 
                                                  current_chordal_notes, 
                                                  strong_beats, 
                                                  current_melodic_range, 
                                                  final_pb, 
                                                  previous_octave)
    
    return (current_chordal_notes, 
            current_scale, 
            current_melodic_range, 
            num_beats, 
            subdivision, 
            strong_beats, 
            df, 
            note_range, 
            bar_notes_range, 
            first_note, 
            last_note, 
            intermediate_melody)

def new_melodic_material(key,
                         original_chord, 
                         original_note, 
                         destination_note, 
                         current_chordal_notes, 
                         current_melodic_range, 
                         note_range, 
                         strong_beats, 
                         last_note, 
                         destination_is_last):

    (possible_melody, 
     beats_in_span) = span_melody_generation(original_note, 
                                             destination_note, 
                                             current_chordal_notes, 
                                             current_melodic_range, 
                                             note_range, 
                                             strong_beats)

    dis = distance_to_destination(possible_melody, 
                                  current_melodic_range, 
                                  destination_note)
    
    final_melody = fit_destination_note(possible_melody, 
                                        dis, 
                                        note_range, 
                                        last_note, 
                                        destination_is_last)

    (variation, 
     stability, 
     jumps, 
     similar_density, 
     span_stability) = melody_weighting(key, 
                                        original_chord, 
                                        final_melody, 
                                        dis, 
                                        current_melodic_range)
    
    return (final_melody, 
            beats_in_span, 
            variation, 
            stability, 
            jumps, 
            similar_density, 
            span_stability)

def new_melodic_generation(key, 
                           current_chord, 
                           possible_octaves, 
                           time_sig, 
                           character, 
                           final_pb, 
                           previous_octave):

    # Generating the skeleton of the first chord
    accept_theme = False
    while not accept_theme:
        (current_chordal_notes, 
        current_scale, 
        current_melodic_range, 
        num_beats, 
        subdivision, 
        strong_beats, 
        df, 
        note_range, 
        bar_notes_range, 
        first_note, 
        last_note, 
        intermediate_melody) = new_melodic_components(key, 
                                                      current_chord, 
                                                      possible_octaves, 
                                                      time_sig, 
                                                      character, 
                                                      final_pb, 
                                                      previous_octave)
        melodic_skeleton = intermediate_melody
        melodic_skeleton.insert(0, first_note)
        melodic_skeleton.append(last_note)
               
        melodic_theme = []
        melodic_beats_in_span = []
        melodic_stability = []
            
        # Generating melodic material
        note_counter = 2
        for skel_idx in range(len(melodic_skeleton)-1):
            original_note = melodic_skeleton[skel_idx]
            destination_note = melodic_skeleton[skel_idx+1]
            span_melodies = []
            weightings = []        
            stabilities = []

            if note_counter == len(melodic_skeleton):
                destination_is_last = True
            else:
                destination_is_last = False

            for iteration in range(100):
                (span_melody, 
                beats_in_span, 
                variation, 
                stability, 
                jumps, 
                similar_density, 
                span_stability) = new_melodic_material(key,
                                                       current_chord,
                                                       original_note, 
                                                       destination_note,
                                                       current_chordal_notes, 
                                                       current_melodic_range, 
                                                       note_range, 
                                                       strong_beats, 
                                                       last_note, 
                                                       destination_is_last)
                span_melodies.append(span_melody)
                weightings.append(variation
                                  +stability
                                  +jumps
                                  +similar_density)
                stabilities.append(span_stability)
            melodic_beats_in_span.append(beats_in_span)
            weightings_n = [weighting/sum(weightings) 
                            for weighting in weightings]
            theme_idx = np.random.choice([mel_idx 
                                          for mel_idx 
                                          in range(len(span_melodies))], 
                                         p=weightings_n)
            melodic_theme.append(span_melodies[theme_idx])
            melodic_stability.append(stabilities[theme_idx])
            note_counter += 1

        flat_melodic_theme = [item for sublist in melodic_theme for item in sublist]
        intervals = [abs(current_melodic_range.index(flat_melodic_theme[note_idx])
                         - current_melodic_range.index(flat_melodic_theme[note_idx+1])) 
                     for note_idx in range(len(flat_melodic_theme)-1)]

        if (all(interval 
               <= 7 for interval in intervals) 
              and all(len(melodic_theme[span_idx])*.25 < melodic_beats_in_span[span_idx]
                       for span_idx in range(len(melodic_theme)))):
            accept_theme = True

    return (melodic_skeleton, 
            melodic_beats_in_span, 
            melodic_theme, 
            melodic_stability)

def new_subordinate(key, 
                    current_chord, 
                    possible_octaves, 
                    time_sig, 
                    dominating_chord,
                    dominating_melody, 
                    dominating_rhythm, 
                    left_subordinate, 
                    bars_that_last):
    
    (current_chordal_notes, 
     current_scale, 
     current_melodic_range) = chord_data(key, 
                                         current_chord, 
                                         possible_octaves)
    
    
    (num_beats, 
     subdivision, 
     strong_beats, 
     reference_dur) = time_sig_data(time_sig)  
    

    # duration = minimum of the dominating span
    current_duration = min([min(span) for span in dominating_rhythm])  
    if num_beats == 4:
        rests = np.random.choice([1,2], p = [.75, .25])    # in beats
    else:
        rests = 1    # in beats
    num_remaining_beats = (num_beats*bars_that_last) - rests
    num_notes = int(num_remaining_beats/current_duration)
    
    if time_sig.split('/')[-1] == str(4):
        ref_dur = 1.0
    else:
        ref_dur = 1.5   
    
    subordinate_melody = [['rest'] for rest in range(rests)]
    subordinate_rhythm = [[ref_dur] for rest in range(rests)]   
    
    if left_subordinate:
        dominating_note = dominating_melody[0][0]
        dominating_scale = chord_data(key, 
                                      dominating_chord, 
                                      possible_octaves)[2]
        dominating_idx = dominating_scale.index(dominating_note)        
        # ascending melodies
        possible_current_first_notes = [note 
                                        for note in current_melodic_range 
                                        if note[:-1] in current_chordal_notes 
                                        and current_melodic_range.index(note) 
                                        <= dominating_idx]
        
        first_idxs = [current_melodic_range.index(note) 
                      for note in possible_current_first_notes]
        
        possible_melodies = [current_melodic_range[first_idx:dominating_idx] 
                             for first_idx in first_idxs 
                             if len(current_melodic_range[first_idx:dominating_idx]) 
                             <= num_notes 
                             and dominating_idx != first_idx]

    else:
        dominating_note = dominating_melody[-1][-1]
        dominating_scale = chord_data(key, 
                                      dominating_chord, 
                                      possible_octaves)[2]
        reversed_dominating_scale = [rev_note 
                                     for rev_note in dominating_scale[::-1]]
        dominating_idx = reversed_dominating_scale.index(dominating_note)
        
        # descending melodies
        reversed_current_melodic_range = [rev_note 
                                  for rev_note in current_melodic_range[::-1]]        
        possible_current_last_notes = [note 
                                        for note in reversed_current_melodic_range 
                                        if note[:-1] in current_chordal_notes 
                                        and reversed_current_melodic_range.index(note) 
                                        >= dominating_idx]    
        last_idxs = [reversed_current_melodic_range.index(note) 
                     for note in possible_current_last_notes]
        possible_melodies = [reversed_current_melodic_range[dominating_idx:last_idx+1]
                             for last_idx in last_idxs
                             if len(reversed_current_melodic_range[dominating_idx:last_idx+1]) 
                             <= num_notes 
                             and dominating_idx != last_idx]
        
    if len(possible_melodies) == 0:
        all_possible_notes = [note 
                              for note in current_melodic_range 
                              if note[:-1] in current_chordal_notes] 
        if dominating_note in current_melodic_range:
            closest_to_dominating = [abs(current_melodic_range.index(dominating_note) 
                                          - current_melodic_range.index(note)) 
                                      for note in all_possible_notes]
            min_idx = closest_to_dominating.index(min(closest_to_dominating))
            possible_melodies.append([all_possible_notes[min_idx]])
        else:
            natural_pitch_class = dominating_note[0]
            pitch_height = dominating_note[-1]
            closest_option = [note 
                              for note in current_melodic_range 
                              if note[0] == natural_pitch_class 
                              and note[-1] == pitch_height]
            possible_melodies = [closest_option]
        
    possible_melodies_lens = [len(poss_mel) 
                              for poss_mel in possible_melodies]
    subordinate_mel_idx = [sub_idx for sub_idx in range(len(possible_melodies))]
    '''
    subordinate_melody.append(possible_melodies[np.random.choice(subordinate_mel_idx, 
                                                                 p = [mel_len/sum(possible_melodies_lens) 
                                                                      for mel_len 
                                                                      in possible_melodies_lens])])
    '''
    sub_mel_probs = [1.0 
                     if mel_len == max(possible_melodies_lens) 
                     else 0.0 
                     for mel_len in possible_melodies_lens]
    sub_mel_probs = [p/sum(sub_mel_probs)
                    for p in sub_mel_probs]
    sub_melodic_choice = possible_melodies[np.random.choice(subordinate_mel_idx, 
                                                            p=sub_mel_probs)]
    
    durations_palette = [.25,.5,.75,1.,1.5,2.,3.,4.,6.]

    if len(sub_melodic_choice) == 1:
        subordinate_melody.append(sub_melodic_choice)
        subordinate_rhythm.append([num_remaining_beats*ref_dur])
        if not left_subordinate:
            subordinate_melody = subordinate_melody[::-1]
            subordinate_rhythm = subordinate_rhythm[::-1]
    elif left_subordinate:
        ascending_rhythm = [current_duration 
                            for dur in range(len(sub_melodic_choice)-1)]
        first_note_duration = [num_remaining_beats*ref_dur - 
                               (len(sub_melodic_choice)-1)*current_duration]
        if first_note_duration[0] not in durations_palette:
            extra_note_dur = first_note_duration[0]%reference_dur
            first_note_duration = first_note_duration[0] - extra_note_dur
            if extra_note_dur != 0.0:
                subordinate_rhythm.append([first_note_duration])
                subordinate_rhythm.append([extra_note_dur] 
                                          + ascending_rhythm)
                subordinate_melody.append([sub_melodic_choice[0]])
            else:
                subordinate_rhythm.append([first_note_duration 
                                           + extra_note_dur]
                                          +ascending_rhythm)      
        else:
            subordinate_rhythm.append(first_note_duration
                                  +ascending_rhythm)
        subordinate_melody.append(sub_melodic_choice)

    else: 
        descending_rhythm = [current_duration 
                             for dur in range(len(sub_melodic_choice)-1)]
        last_note_duration = [num_remaining_beats*ref_dur - 
                               (len(sub_melodic_choice)-1)*current_duration]
        if last_note_duration[0] not in durations_palette:
            extra_note_dur = last_note_duration[0]%reference_dur
            last_note_duration = last_note_duration[0] - extra_note_dur
            #subordinate_melody.append([subordinate_melody[-1][0]])
            if extra_note_dur != 0.0:
                subordinate_rhythm.append([last_note_duration])
                subordinate_rhythm.append(descending_rhythm 
                                          + [extra_note_dur])
                subordinate_melody.append([sub_melodic_choice[-1]])

            else:
                subordinate_rhythm.append(descending_rhythm
                                          + [last_note_duration]
                                          + [extra_note_dur])   
                subordinate_melody.append([sub_melodic_choice[-1]])

        else:
            subordinate_rhythm.append(descending_rhythm
                                      + last_note_duration)
        subordinate_melody.append(sub_melodic_choice)

        subordinate_melody = subordinate_melody[::-1]
        subordinate_rhythm = subordinate_rhythm[::-1]
    
    return subordinate_melody, subordinate_rhythm

    
    
    