

import numpy as np
import math
from collections import Counter
from music21 import pitch
from util_functions.melodic_components import chord_data
from AuToTen.setup import classes

def melodic_transposition(current_chord, 
                          reference_chord, 
                          key, 
                          possible_octaves, 
                          reference_melody, 
                          reference_rhythm):
    
    (reference_chordal_notes, 
     reference_scale, 
     reference_range) = chord_data(key, 
                                   reference_chord, 
                                   possible_octaves)
    if any(x not in reference_range 
           for x in reference_melody[0]):
        new_ref_melody = []
        for n in reference_melody[0]:
            if n in reference_range:
                new_ref_melody.append(n)
            elif n[0]+'#'+n[-1] in reference_range:
                new_ref_melody.append(n[0]+'#'+n[-1])
            elif n[0]+'b'+n[-1] in reference_range:
                new_ref_melody.append(n[0]+'b'+n[-1])                
        reference_melody = [new_ref_melody]
        
    reference_root = (reference_chordal_notes[0]
                      +reference_melody[0][0][-1])
    
    (current_chordal_notes, 
     current_scale, 
     current_range) = chord_data(key, 
                                 current_chord, 
                                 possible_octaves)

    current_root = (current_chordal_notes[0]
                      +reference_melody[0][0][-1]) 
    root_idx = current_range.index(current_root)
    current_range = (current_range[root_idx:] 
                     + current_range[:root_idx])
    
    transposed_melody = []
    for group in reference_melody:
        trans_group = []
        for note in group:
            distance_to_ref = (reference_range.index(reference_root) 
                               - reference_range.index(note))
            note_idx = (current_range.index(current_root) 
                        - distance_to_ref)%(len(current_range)-1)
            trans_group.append(current_range[note_idx])    
        transposed_melody.append(trans_group)
    
    return transposed_melody, reference_rhythm

def steps_calculator(current_chord, 
                     reference_chord, 
                     key,
                     possible_octaves, 
                     reference_melody, 
                     reference_rhythm):
    
    (reference_chordal_notes, 
     reference_scale, 
     reference_range) = chord_data(key, 
                                   reference_chord, 
                                   possible_octaves)
    
    reference_root = (reference_chordal_notes[0]
                      +reference_melody[0][0][-1])
        

    (current_chordal_notes, 
     current_scale, 
     current_range) = chord_data(key, 
                                 current_chord, 
                                 possible_octaves)

    current_root = (current_chordal_notes[0]
                    +reference_melody[0][0][-1])    
    
    # Calculate distance between roots
    roots_distance = (reference_range.index(reference_root)
                      -current_range.index(current_root))
    
    return (reference_range, 
            roots_distance, 
            current_range, 
            reference_root,
            current_root)

def range_modulo(new_idx, current_range):
    
    if new_idx >= len(current_range):
        return new_idx - 7
    elif new_idx < 0:
        return  new_idx + 7
    else:
        return new_idx

def melodic_inversion(current_chord,
                      reference_chord, 
                      key, 
                      possible_octaves, 
                      reference_melody, 
                      reference_rhythm):
    
    transposed_melody = melodic_transposition(current_chord, 
                                              reference_chord, 
                                              key, 
                                              possible_octaves, 
                                              reference_melody, 
                                              reference_rhythm)[0]

    current_range = chord_data(key, 
                               current_chord, 
                               possible_octaves)[2]
    
    
    
    current_melody = []
    for group in transposed_melody:
        reference_note = group[0]
        group_melody = []
        for note in group:
            distance_to_ref = (current_range.index(reference_note) 
                               - current_range.index(note))
            note_idx = (current_range.index(reference_note) 
                        + distance_to_ref)%(len(current_range)-1)
            group_melody.append(current_range[note_idx])
        
        current_melody.append(group_melody)
    return current_melody, reference_rhythm    

def inverted_consequence(key, 
                         current_chord, 
                         reference_chord, 
                         possible_octaves, 
                         reference_melody, 
                         reference_rhythm, 
                         descending):
    (current_melody, 
     current_rhythm) = melodic_inversion(current_chord,
                                         reference_chord, 
                                         key, 
                                         possible_octaves, 
                                         reference_melody, 
                                         reference_rhythm)
    current_range = steps_calculator(current_chord, 
                                      reference_chord,
                                      key, 
                                      possible_octaves,
                                      reference_melody, 
                                      reference_rhythm)[2]    
    (consequent_melody, 
     consequent_rhythm) = consequent_variation(key, 
                                               current_chord, 
                                               possible_octaves, 
                                               current_melody, 
                                               current_rhythm, 
                                               current_range, 
                                               descending)


    return consequent_melody, consequent_rhythm


def consequent_variation(key, 
                         current_chord, 
                         possible_octaves, 
                         current_melody, 
                         current_rhythm, 
                         current_range, 
                         descending):
    
    current_chordal_notes = chord_data(key, 
                                   current_chord, 
                                   possible_octaves)[0]
    reference_octave = current_melody[-1][0][-1]
    root = current_chordal_notes[0]+reference_octave
    root_idx = current_range.index(root)
    
    '''
    Consequent code is provided but is not applied in the current implementation.
    Uncomment the last line to apply it.
    '''
    # A good consequent:
    # ---> ascending direction if "norm" (right branch)
    # ---> descending direction if "dom" (left branch)
    # ---> ends with root
    
    new_span_melody = []
    new_note_idx = root_idx
    for new_note in range(len(current_melody[-1])):
        new_span_melody.append(current_range[new_note_idx])
        new_note_idx += 1
        new_note_idx = range_modulo(new_note_idx, current_range)
    
    if descending:
        new_span_melody = new_span_melody[::-1]
    new_span_rhythm = tuple(sorted(current_rhythm[-1]))
    
    consequent_melody = [current_melody[span] 
                         for span in range(len(current_melody)-1)]
    consequent_melody.append(new_span_melody)

    consequent_rhythm =  [current_rhythm[span] 
                          for span in range(len(current_rhythm)-1)]
    consequent_rhythm.append(new_span_rhythm)

    return consequent_melody, consequent_rhythm
    #return new_span_melody, new_span_rhythm

def last_chord(first_melodic_theme, 
               first_melodic_rhythm, 
               first, 
               last, 
               key, 
               possible_octaves, 
               strong_beats, 
               reference_dur, 
               dom_octave):
              
    (current_chordal_notes, 
     scale, 
     current_melodic_range) = chord_data(key,
                                         last,
                                         possible_octaves)
    repeat = np.random.choice([True, False], p=[.7,.3])
    
    if repeat:       
        last_theme = first_melodic_theme
        last_rhythm = first_melodic_rhythm
        last_note = last_theme[-1][-1]
        if (last_note[:-1] not in current_chordal_notes):
            distance_to_last_note = [abs(idx-current_melodic_range.index(last_note)) 
                                     if current_melodic_range[idx][:-1] 
                                     in current_chordal_notes else math.inf
                                     for idx in range(len(current_melodic_range))]
            last_note = current_melodic_range[distance_to_last_note.index(min(distance_to_last_note))]
            last_theme[-1][-1] = last_note
    else:
        spans_dur = [dur 
                        for group in first_melodic_rhythm 
                        for dur in group]
        last_note = np.random.choice(current_chordal_notes)
        last_note = last_note+dom_octave
        last_theme = [[last_note]]
        last_rhythm = [[sum(spans_dur)]]
        
        
    return last_theme, last_rhythm
    
def melodic_retrogression(current_chord,
                          reference_chord, 
                          key, 
                          possible_octaves, 
                          reference_melody, 
                          reference_rhythm):
    
    transposed_melody = melodic_transposition(current_chord, 
                                              reference_chord, 
                                              key, 
                                              possible_octaves, 
                                              reference_melody, 
                                              reference_rhythm)[0]
    current_melody = []
    current_rhythm = []
    retro_melody = transposed_melody[::-1]
    retro_rhythm = reference_rhythm[::-1]
    for group in range(len(retro_melody)):
        current_melody.append(retro_melody[group][::-1])
        current_rhythm.append(retro_rhythm[group][::-1])
            
    return current_melody, current_rhythm           
    
