

import numpy as np
from music21 import note, stream, key, tempo, chord, instrument, pitch, meter
from AuToTen.setup import classes            

def pb_stream(param_key, 
              bpm, 
              time_sig, 
              final_pb, 
              first_melodic_theme, 
              first_melodic_rhythm, 
              rep_melodic_theme, 
              rep_melodic_rhythm, 
              other_melodic_theme, 
              other_melodic_rhythm, 
              norm_melodic_theme, 
              norm_melodic_rhythm, 
              left_subother_melodic_theme, 
              left_subother_melodic_rhythm, 
              left_subnorm_melodic_theme, 
              left_subnorm_melodic_rhythm, 
              right_subother_melodic_theme, 
              right_subother_melodic_rhythm, 
              right_subnorm_melodic_theme, 
              right_subnorm_melodic_rhythm, 
              reference_dur):
    
    # Initialize music21's stream
    s = stream.Stream()
    s.keySignature = key.Key(param_key)
    t = tempo.MetronomeMark(bpm)
    t.setQuarterBPM(bpm*reference_dur)
    s.append(t)
    s.append(meter.TimeSignature(time_sig))

    sv = stream.Stream()
    sv.keySignature = key.Key(param_key)
    sv.append(t)
    sv.append(meter.TimeSignature(time_sig))
    
    notes = []
    durations = []
    for chord in final_pb:
        if chord == 'first':
            chord_notes = list(np.concatenate(first_melodic_theme))
            #chord_notes = first_melodic_theme
            chord_durations = list(np.concatenate(first_melodic_rhythm))
        elif chord == 'rep':
            chord_notes = list(np.concatenate(rep_melodic_theme))
            #chord_notes = rep_melodic_theme
            chord_durations = list(np.concatenate(rep_melodic_rhythm))            
        elif chord == 'other':
            chord_notes = list(np.concatenate(other_melodic_theme))
            #chord_notes = other_melodic_theme
            chord_durations = list(np.concatenate(other_melodic_rhythm))            
        elif chord == 'norm':
            chord_notes = list(np.concatenate(norm_melodic_theme))
            #chord_notes = norm_melodic_theme
            chord_durations = list(np.concatenate(norm_melodic_rhythm))            
        elif chord == '*other':
            chord_notes = list(np.concatenate(left_subother_melodic_theme))
            #chord_notes = left_subother_melodic_theme
            chord_durations = list(np.concatenate(left_subother_melodic_rhythm))            
        elif chord == 'other*':
            chord_notes = list(np.concatenate(right_subother_melodic_theme))
            #chord_notes = right_subother_melodic_theme
            chord_durations = list(np.concatenate(right_subother_melodic_rhythm))            
        elif chord == '*norm':
            chord_notes = list(np.concatenate(left_subnorm_melodic_theme))
            #chord_notes = left_subnorm_melodic_theme
            chord_durations = list(np.concatenate(left_subnorm_melodic_rhythm))            
        elif chord == 'norm*':
            chord_notes = list(np.concatenate(right_subnorm_melodic_theme))
            #chord_notes = right_subnorm_melodic_theme
            chord_durations = list(np.concatenate(right_subnorm_melodic_rhythm))
        
        if type(chord) != float:
            notes.append(chord_notes)
            durations.append(chord_durations)
    flat_notes = list(np.concatenate(notes))
    flat_durations = list(np.concatenate(durations))
    for note_idx in range(len(flat_notes)):
        if flat_notes[note_idx] == 'rest':
            n = note.Rest(quarterLength=flat_durations[note_idx])
        elif '##' in flat_notes[note_idx]:
            enh = pitch.Pitch(flat_notes[note_idx])
            n = note.Note(enh.getEnharmonic(), 
                          quarterLength=flat_durations[note_idx])
        elif ('bb' in flat_notes[note_idx] 
              and len(flat_notes[note_idx]) == 4):
            enh = pitch.Pitch(flat_notes[note_idx])
            n = note.Note(enh.getEnharmonic(), 
                          quarterLength=flat_durations[note_idx])
        else:
            n = note.Note(flat_notes[note_idx], 
                          quarterLength=flat_durations[note_idx])
        s.append(n)
        sv.append(n)
    return s, sv

def harm_seq(param_key, 
             pb_chords, 
             time_sig, 
             bpm, 
             reference_dur):
    
    chords_notes = []
    chords_durations = []
    div, subdiv = time_sig.split('/')
    if subdiv ==  str(8):
        num_16ths = 6
        div = int(div)/3
    else:
        num_16ths = 4
    bar_duration = num_16ths*.25*int(div)
    current_duration = bar_duration
    for current_chord in pb_chords:
        if type(current_chord) != float:
            # Calculating chordal notes
            chordal_notes = [classes.notes.Notes(param_key, 
                                                 current_chord).root(), 
                             classes.notes.Notes(param_key, 
                                                 current_chord).third(),
                             classes.notes.Notes(param_key, 
                                                 current_chord).fifth(),
                             classes.notes.Notes(param_key, 
                                                 current_chord).seventh()]    

            if not chordal_notes[-1]:
                chordal_notes[-1] = chordal_notes[0]
                
            chordal_notes = [c_note[0].upper()
                             +c_note[1:] 
                             for c_note in chordal_notes]
            #chordal_notes = [note+str(3) for note in chordal_notes]
            #chordal_notes[-1] = chordal_notes[-1][:-1]+str(4)
            chordal_notes[0] = chordal_notes[0]+str(3)
            chordal_notes[1] = chordal_notes[1]+str(3)
            chordal_notes[2] = chordal_notes[2]+str(4)
            chordal_notes[3] = chordal_notes[3]+str(4)               
            chords_notes.append(chordal_notes)
            chords_durations.append(bar_duration)
        else:
            chords_durations[-1] = bar_duration*2
    bass = stream.Voice()
    bass.keySignature = key.Key(param_key)
    tenor = stream.Voice()
    tenor.keySignature = key.Key(param_key)
    alto = stream.Voice()
    alto.keySignature = key.Key(param_key)
    soprano = stream.Voice()
    soprano.keySignature = key.Key(param_key)
    t = tempo.MetronomeMark(bpm)
    t.setQuarterBPM(bpm*reference_dur)
    bass.append(t)
    tenor.append(t)
    alto.append(t)
    soprano.append(t)
    bass.append(meter.TimeSignature(time_sig))
    tenor.append(meter.TimeSignature(time_sig))
    alto.append(meter.TimeSignature(time_sig))
    soprano.append(meter.TimeSignature(time_sig))
    for bar in range(len(chords_notes)):
        bass.append(note.Note(chords_notes[bar][0], 
              quarterLength=chords_durations[bar]))
        tenor.append(note.Note(chords_notes[bar][3], 
              quarterLength=chords_durations[bar]))
        alto.append(note.Note(chords_notes[bar][1], 
              quarterLength=chords_durations[bar]))
        soprano.append(note.Note(chords_notes[bar][2], 
              quarterLength=chords_durations[bar]))
    return bass, tenor, alto, soprano

def midi_gen(mel_voice,
            soprano,
            alto,
            tenor,
            bass, 
            reference_dur):

    midi_file = stream.Stream()
    midi_file.insert(0, mel_voice)
    midi_file.insert(0, soprano)
    midi_file.insert(0, alto)
    midi_file.insert(0, tenor)
    midi_file.insert(0, bass)
    
    return midi_file

def cs_stream(param_key, 
              bpm, 
              time_sig, 
              final_cs,
              last_melodic_theme, 
              last_melodic_rhythm, 
              dom_melodic_theme, 
              dom_melodic_rhythm, 
              sub_melodic_theme, 
              sub_melodic_rhythm, 
              cad_melodic_theme, 
              cad_melodic_rhythm, 
              sub_1_melodic_theme, 
              sub_1_melodic_rhythm, 
              sub_2_melodic_theme, 
              sub_2_melodic_rhythm, 
              sub_3_melodic_theme, 
              sub_3_melodic_rhythm, 
              reference_dur):
    
    # Initialize music21's stream
    s = stream.Stream()
    s.keySignature = key.Key(param_key)
    t = tempo.MetronomeMark(bpm)
    t.setQuarterBPM(bpm*reference_dur)
    s.append(t)
    s.append(meter.TimeSignature(time_sig))

    sv = stream.Stream()
    sv.keySignature = key.Key(param_key)
    sv.append(t)
    sv.append(meter.TimeSignature(time_sig))
    
    notes = []
    durations = []
    for chord in final_cs:
        if chord == 'last':
            chord_notes = list(np.concatenate(last_melodic_theme))
            #chord_notes = first_melodic_theme
            chord_durations = list(np.concatenate(last_melodic_rhythm))
        elif chord == 'dom':
            chord_notes = list(np.concatenate(dom_melodic_theme))
            #chord_notes = rep_melodic_theme
            chord_durations = list(np.concatenate(dom_melodic_rhythm))            
        elif chord == 'sub':
            chord_notes = list(np.concatenate(sub_melodic_theme))
            #chord_notes = other_melodic_theme
            chord_durations = list(np.concatenate(sub_melodic_rhythm))            
        elif chord == 'cad64':
            chord_notes = list(np.concatenate(cad_melodic_theme))
            #chord_notes = norm_melodic_theme
            chord_durations = list(np.concatenate(cad_melodic_rhythm))            
        elif chord == 'sub*':
            chord_notes = list(np.concatenate(sub_1_melodic_theme))
            #chord_notes = left_subother_melodic_theme
            chord_durations = list(np.concatenate(sub_1_melodic_rhythm))            
        elif chord == 'sub**':
            chord_notes = list(np.concatenate(sub_2_melodic_theme))
            #chord_notes = right_subother_melodic_theme
            chord_durations = list(np.concatenate(sub_2_melodic_rhythm))            
        elif chord == 'sub***':
            chord_notes = list(np.concatenate(sub_3_melodic_theme))
            #chord_notes = left_subnorm_melodic_theme
            chord_durations = list(np.concatenate(sub_3_melodic_rhythm))            
        
        if type(chord) != float:
            notes.append(chord_notes)
            durations.append(chord_durations)
    flat_notes = list(np.concatenate(notes))
    flat_durations = list(np.concatenate(durations))
    for note_idx in range(len(flat_notes)):
        if flat_notes[note_idx] == 'rest':
            n = note.Rest(quarterLength=flat_durations[note_idx])
        elif '##' in flat_notes[note_idx]:
            enh = pitch.Pitch(flat_notes[note_idx])
            n = note.Note(enh.getEnharmonic(), 
                          quarterLength=flat_durations[note_idx])
        elif ('bb' in flat_notes[note_idx] 
              and len(flat_notes[note_idx]) == 4):
            enh = pitch.Pitch(flat_notes[note_idx])
            n = note.Note(enh.getEnharmonic(), 
                          quarterLength=flat_durations[note_idx])
        else:
            n = note.Note(flat_notes[note_idx], 
                          quarterLength=flat_durations[note_idx])
        s.append(n)
        sv.append(n)
    return s, sv

def final_theme(pb_mel_voice, 
                cs_mel_voice, 
                pb_bass, 
                pb_tenor, 
                pb_alto, 
                pb_soprano, 
                cs_bass, 
                cs_tenor, 
                cs_alto, 
                cs_soprano):
    
    offsets = [note.offset for note in pb_mel_voice]
    durations = [note.duration.quarterLength for note in pb_mel_voice]
    dur_diff = offsets[-1]+durations[-1]
    cs_offsets = [note.offset for note in cs_mel_voice]
    
    new_mel = cs_mel_voice
    for note in new_mel:
        note.offset += dur_diff
    new_bass = cs_bass
    for note in new_bass:
        note.offset += dur_diff        
    new_tenor = cs_tenor
    for note in new_tenor:
        note.offset += dur_diff        
    new_alto = cs_alto
    for note in new_alto:
        note.offset += dur_diff  
    new_soprano = cs_soprano
    for note in new_soprano:
        note.offset += dur_diff        
        
    final_mel = pb_mel_voice + new_mel
    final_bass = pb_bass + new_bass
    final_tenor = pb_tenor + new_tenor
    final_alto = pb_alto + new_alto
    final_soprano = pb_soprano + new_soprano
    
    return (final_mel, 
            final_bass, 
            final_tenor, 
            final_alto, 
            final_soprano)