

import numpy as np
from util_functions.melodic_sequence import new_melodic_generation, new_subordinate
from util_parameters.melodic_param import possible_octaves
from util_functions.melodic_rhythm import rhythm_assignment
from util_functions.melodic_transformations import melodic_transposition, inverted_consequence

def pb_theme(final_pb, 
             key, 
             first, 
             rep, 
             other, 
             norm, 
             left_subother, 
             right_subother,
             left_subnorm,
             right_subnorm,
             time_sig, 
             character):
    
    # Generating "first"
    (first_melodic_skeleton, 
    first_melodic_beats_in_span, 
    first_melodic_theme, 
    first_melodic_stability) = new_melodic_generation(key, 
                                                      first, 
                                                      possible_octaves, 
                                                      time_sig, 
                                                      character, 
                                                      final_pb, 
                                                      False)
    first_melodic_rhythm = rhythm_assignment(time_sig, 
                                             first_melodic_theme, 
                                             first_melodic_beats_in_span, 
                                             first_melodic_stability)
    previous_octave = first_melodic_theme[-1][-1][-1]
    
    # Generating "rep"
    if 'rep' in final_pb:
        if np.random.choice([True,False]):
            # new material
            (rep_melodic_skeleton, 
            rep_melodic_beats_in_span, 
            rep_melodic_theme, 
            rep_melodic_stability) = new_melodic_generation(key, 
                                                            rep, 
                                                            possible_octaves, 
                                                            time_sig, 
                                                            character, 
                                                            final_pb, 
                                                            previous_octave)
            rep_melodic_rhythm = rhythm_assignment(time_sig, 
                                                  rep_melodic_theme, 
                                                  rep_melodic_beats_in_span, 
                                                  rep_melodic_stability)

        else:
            # first's transposition
            (rep_melodic_theme, 
             rep_melodic_rhythm) = melodic_transposition(rep, 
                                                         first, 
                                                         key, 
                                                         possible_octaves, 
                                                         first_melodic_theme, 
                                                         first_melodic_rhythm)
        previous_octave = rep_melodic_theme[-1][-1][-1]
    else:
        rep_melodic_theme = None
        rep_melodic_rhythm = None

    # Generating "other" 
    if 'other' in final_pb:
        (other_melodic_skeleton, 
        other_melodic_beats_in_span, 
        other_melodic_theme, 
        other_melodic_stability) = new_melodic_generation(key, 
                                                          other, 
                                                          possible_octaves, 
                                                          time_sig, 
                                                          character, 
                                                          final_pb, 
                                                          previous_octave)
        other_melodic_rhythm = rhythm_assignment(time_sig, 
                                                 other_melodic_theme, 
                                                 other_melodic_beats_in_span, 
                                                 other_melodic_stability)
    else:
        other_melodic_theme = None
        other_melodic_rhythm = None    
        
    # Generating "norm"      
    if 'other' in final_pb:
        reference_is_first = np.random.choice([True,False])
        if not reference_is_first:
            reference_chord = other
            ref_melodic_theme = other_melodic_theme
            ref_melodic_rhythm = other_melodic_rhythm
    else:
        reference_is_first = True
    if reference_is_first:
        reference_chord = first
        ref_melodic_theme = first_melodic_theme
        ref_melodic_rhythm = first_melodic_rhythm

    (norm_melodic_theme, 
     norm_melodic_rhythm) = inverted_consequence(key, 
                                                 norm, 
                                                 reference_chord, 
                                                 possible_octaves, 
                                                 ref_melodic_theme, 
                                                 ref_melodic_rhythm, 
                                                 False)
    
    # Generating "subordinates"
    if '*other' in final_pb:
        bars_that_last = lasting_bars(final_pb, '*other', 'other')
        (left_subother_melodic_theme, 
         left_subother_melodic_rhythm) = new_subordinate(key, 
                                                         left_subother, 
                                                         possible_octaves, 
                                                         time_sig, 
                                                         other,
                                                         other_melodic_theme, 
                                                         other_melodic_rhythm, 
                                                         True, 
                                                         bars_that_last) 
    else:
        left_subother_melodic_theme = None
        left_subother_melodic_rhythm = None       
        
    if '*norm' in final_pb:
        bars_that_last = lasting_bars(final_pb, '*norm', 'norm')
        (left_subnorm_melodic_theme, 
         left_subnorm_melodic_rhythm) = new_subordinate(key, 
                                                        left_subnorm, 
                                                        possible_octaves, 
                                                        time_sig, 
                                                        norm,
                                                        norm_melodic_theme, 
                                                        norm_melodic_rhythm, 
                                                        True, 
                                                        bars_that_last) 
    else:
        left_subnorm_melodic_theme = None
        left_subnorm_melodic_rhythm = None 
        
    if 'other*' in final_pb:
        bars_that_last = lasting_bars(final_pb, 'other*', 'other')
        (right_subother_melodic_theme, 
         right_subother_melodic_rhythm) = new_subordinate(key, 
                                                          right_subother, 
                                                          possible_octaves, 
                                                          time_sig, 
                                                          other,
                                                          other_melodic_theme, 
                                                          other_melodic_rhythm, 
                                                          False, 
                                                          bars_that_last) 

    else:
        right_subother_melodic_theme = None
        right_subother_melodic_rhythm = None 
        
    if 'norm*' in final_pb:
        bars_that_last = lasting_bars(final_pb, 'norm*', 'norm')
        (right_subnorm_melodic_theme, 
         right_subnorm_melodic_rhythm) = new_subordinate(key, 
                                                         right_subnorm, 
                                                         possible_octaves, 
                                                         time_sig, 
                                                         norm,
                                                         norm_melodic_theme, 
                                                         norm_melodic_rhythm, 
                                                         False, 
                                                         bars_that_last)         
    else:        
        right_subnorm_melodic_theme = None
        right_subnorm_melodic_rhythm = None 
    
    
    # Optimization
    original_first = original_theme(first_melodic_theme)
    first_melodic_theme = theme_optimization(first_melodic_theme, 
                                             key, 
                                             first, 
                                             possible_octaves)
    if rep:
        original_rep = original_theme(rep_melodic_theme)
        rep_melodic_theme = theme_optimization(rep_melodic_theme, 
                                               key, 
                                               rep, 
                                               possible_octaves)
    else:
        original_rep = None
    if other:
        original_other = original_theme(other_melodic_theme)
        other_melodic_theme = theme_optimization(other_melodic_theme, 
                                                 key, 
                                                 other, 
                                                 possible_octaves)
    else:
        original_other = None
    norm_melodic_theme = theme_optimization(norm_melodic_theme, 
                                            key, 
                                            norm, 
                                            possible_octaves)
    if left_subother:
        left_subother_melodic_theme = theme_optimization(left_subother_melodic_theme, 
                                                         key, 
                                                         left_subother, 
                                                         possible_octaves)
    if left_subnorm:
        left_subnorm_melodic_theme = theme_optimization(left_subnorm_melodic_theme, 
                                                        key, 
                                                        left_subnorm, 
                                                        possible_octaves)
    if right_subother:
        right_subother_melodic_theme = theme_optimization(right_subother_melodic_theme, 
                                                          key, 
                                                          right_subother, 
                                                          possible_octaves)
    if right_subnorm:
        right_subnorm_melodic_theme = theme_optimization(right_subnorm_melodic_theme, 
                                                         key, 
                                                         right_subnorm, 
                                                         possible_octaves)
       
    return (first_melodic_theme, 
            first_melodic_rhythm, 
            rep_melodic_theme, 
            rep_melodic_rhythm, 
            other_melodic_theme, 
            other_melodic_rhythm, 
            norm_melodic_theme, 
            norm_melodic_rhythm, 
            left_subother_melodic_theme, 
            left_subother_melodic_rhythm, 
            left_subnorm_melodic_theme, 
            left_subnorm_melodic_rhythm, 
            right_subother_melodic_theme, 
            right_subother_melodic_rhythm, 
            right_subnorm_melodic_theme, 
            right_subnorm_melodic_rhythm, 
            original_first, 
            original_rep, 
            original_other)

def lasting_bars(final_pb, 
                 subordinate_chord, 
                 dominant_chord):
    
    bars_that_last = abs((final_pb.index(dominant_chord) 
                      - final_pb.index(subordinate_chord)))
    return bars_that_last

from util_functions.melodic_components import chord_data

def theme_optimization(current_theme, 
                       key, 
                       current_chord, 
                       possible_octaves):
    # ensure the melodic scale is used in the minor mode
    flat_theme = [item 
                  for sublist in current_theme 
                  for item in sublist]
    # use the tonic chord
    tonic_chord = 'i/' + current_chord.split('/')[-1]
    chordal_notes, scale, current_melodic_range = chord_data(key, 
                                                             tonic_chord, 
                                                             possible_octaves)
    root = chordal_notes[0]
    root_idx = scale.index(root)
    sorted_scale = scale[root_idx:]+scale[:root_idx]
    seventh_degree = sorted_scale[-1]
    sixth_degree = sorted_scale[-2]
    chordal_notes, scale, current_melodic_range = chord_data(key, 
                                                         current_chord, 
                                                         possible_octaves)
    intervals = []
    for idx in range(len(flat_theme)-1):
        if (flat_theme[idx] != 'rest' 
            and flat_theme[idx+1] != 'rest'):
            intervals.append(abs(current_melodic_range.index(flat_theme[idx])
                                 - current_melodic_range.index(flat_theme[idx+1])))
    seventh_idxs = [idx 
                    for idx in range(len(flat_theme)) 
                    if flat_theme[idx][0] == seventh_degree]
    if current_chord.split('/')[-1].islower():
        for idx in seventh_idxs:
            if (current_chord.split('/')[0] == 'V' 
                and idx != 0 
                and idx != len(flat_theme)-1):
                if (flat_theme[idx] != 'rest' 
                    and flat_theme[idx-1] != 'rest'):
                    if (current_melodic_range.index(flat_theme[idx]) 
                        - current_melodic_range.index(flat_theme[idx-1]) == 1 
                        and flat_theme[idx-1][:-1] == sorted_scale[5] 
                        and flat_theme[idx-1][:-1] not in chordal_notes):                    
                        new_note = alter_up(flat_theme[idx-1], 
                                            sorted_scale) 
                        flat_theme[idx-1] = new_note       
                if (flat_theme[idx] != 'rest' 
                    and flat_theme[idx+1] != 'rest'):
                    if (current_melodic_range.index(flat_theme[idx]) 
                        - current_melodic_range.index(flat_theme[idx+1]) == 1
                        and flat_theme[idx+1][:-1] == sorted_scale[6] 
                        and flat_theme[idx+1][:-1] not in chordal_notes):
                        new_note = alter_up(flat_theme[idx+1], 
                                            sorted_scale) 
                        flat_theme[idx+1] = new_note       
            elif idx+1 < len(flat_theme):    # check the seventh isn't the last note
                # if the seventh goes to the root ---> alter it
                if (flat_theme[idx] != 'rest' 
                    and flat_theme[idx+1] != 'rest'):
                    up_idx = None
                    if (current_melodic_range.index(flat_theme[idx+1]) 
                        - current_melodic_range.index(flat_theme[idx]) == 1 
                        and flat_theme[idx+1][:-1] == root 
                        and flat_theme[idx][:-1] == sorted_scale[6]
                        and flat_theme[idx] not in chordal_notes):
                        up_new_note = alter_up(flat_theme[idx], 
                                            sorted_scale) 
                        up_idx = idx
                    # if the seventh is preceded by the sixth ---> alter it
                    up2_idx = None
                    if idx != 0:
                        if (flat_theme[idx]!= 'rest' 
                            and flat_theme[idx-1] != 'rest'):    # the seventh isn't the first note
                            if (current_melodic_range.index(flat_theme[idx]) 
                                - current_melodic_range.index(flat_theme[idx-1]) == 1 
                                and flat_theme[idx+1][:-1] == root 
                                and flat_theme[idx-1][:-1] == sorted_scale[5]
                                and flat_theme[idx-1] not in chordal_notes):                    
                                up2_new_note = alter_up(flat_theme[idx-1], 
                                                    sorted_scale) 
                                up2_idx = idx-1
                    if up_idx:
                        flat_theme[up_idx] = up_new_note
                    if up2_idx:
                        flat_theme[up2_idx] = up2_new_note
    
    theme_in_groups = []
    note_idx = 0
    for group in current_theme:
        group_notes = []
        for note in group:
            group_notes.append(flat_theme[note_idx])
            note_idx += 1
        theme_in_groups.append(group_notes)
    return theme_in_groups
    
def alter_up(note, sorted_scale):
    
    if '##' in note:
        note_idx = ((sorted_scale.index(note[:-1])
                     +1)%len(sorted_scale))
        new_note = sorted_scale[note_idx]+'#'+note[-1]
    elif 'bb' in note and len(note) == 4:
        new_note = note[:1]+note[-1]
    elif '#' in note:
        new_note = note[:1]+'#'+note[-1]
    elif 'b' in note and len(note) == 3:
        new_note = note[0]+note[-1]
    else:
        new_note = note[0]+'#'+note[-1]
    return new_note

def alter_down(note, sorted_scale):
    
    if '##' in note:
        new_note = note[:1]+note[:-1]
    elif 'bb' in note and len(note) == 4:
        note_idx = ((sorted_scale.index(note[:-1])
                     -1)%len(sorted_scale))
        new_note = sorted_scale[note_idx]+'b'+note[-1]
    elif '#' in note:
        new_note = note[0]+note[-1]
    elif 'b' in note and len(note) == 3:
        new_note = note[:1]+'b'+note[-1]
    else:
        new_note = note[0]+'b'+note[-1]
    return new_note

def original_theme(melodic_theme):
    return [item for item in melodic_theme]

def final_melody_pb(final_pb, 
                    first_melodic_theme, 
                    rep_melodic_theme, 
                    right_subother_melodic_theme, 
                    left_subother_melodic_theme, 
                    other_melodic_theme, 
                    right_subnorm_melodic_theme, 
                    left_subnorm_melodic_theme, 
                    norm_melodic_theme,
                    first_melodic_rhythm, 
                    rep_melodic_rhythm, 
                    right_subother_melodic_rhythm, 
                    left_subother_melodic_rhythm, 
                    other_melodic_rhythm, 
                    right_subnorm_melodic_rhythm, 
                    left_subnorm_melodic_rhythm, 
                    norm_melodic_rhythm):
    
    possible_chords = ['first', 'rep', '*other', 'other', 
                       'other*', '*norm' , 'norm', 'norm*']
    
    final_melody = []
    final_rhythm = []
    for chord in final_pb:
        if type(chord) != float:
            if chord == possible_chords[0]:
                for spans in first_melodic_theme:
                    final_melody.append(spans)
                for spans in first_melodic_rhythm:
                    final_rhythm.append(spans)
            elif chord == possible_chords[1]:
                for spans in rep_melodic_theme:
                    final_melody.append(spans)
                for spans in rep_melodic_rhythm:
                    final_rhythm.append(spans)                    
            elif chord == possible_chords[2]:
                for spans in left_subother_melodic_theme:
                    final_melody.append(spans)
                for spans in left_subother_melodic_rhythm:
                    final_rhythm.append(spans)                    
            elif chord == possible_chords[3]:
                for spans in other_melodic_theme:
                    final_melody.append(spans)
                for spans in other_melodic_rhythm:
                    final_rhythm.append(spans)                    
            elif chord == possible_chords[4]:
                for spans in right_subother_melodic_theme:
                    final_melody.append(spans)
                for spans in right_subother_melodic_rhythm:
                    final_rhythm.append(spans)
            elif chord == possible_chords[5]:
                for spans in left_subnorm_melodic_theme:
                    final_melody.append(spans)
                for spans in left_subnorm_melodic_rhythm:
                    final_rhythm.append(spans)                    
            elif chord == possible_chords[6]:
                for spans in norm_melodic_theme:
                    final_melody.append(spans)
                for spans in norm_melodic_rhythm:
                    final_rhythm.append(spans)                    
            elif chord == possible_chords[7]:
                for spans in right_subnorm_melodic_theme:
                    final_melody.append(spans)
                for spans in right_subnorm_melodic_rhythm:
                    final_rhythm.append(spans)
                    
    return final_melody, final_rhythm
            
