

import csv
import ast
import pandas

from util_functions.melodic_components import chord_data
from util_parameters.melodic_param import possible_octaves
from AuToTen.setup import common, distance, a_calculator, t_calculator

def matrix(final_pb, final_cs, pb_chords, cs_chords):
    
    # Dictionary: [row, column, value]
    labels = {'first':[0,0,0], 'rep':[], 'norm':[], 
              '*norm':[], 'norm*':[], 'other':[], 
              '*other':[], 'other*':[], 'dom':[], 
              'last':[], 'cad64':[], 'sub':[], 
              'sub*':[], 'sub**':[], 'sub***':[]}

    # Populating the labels
    idx = 0
    real_pb = [real_chord 
           for real_chord in final_pb
           if type(real_chord) != float]
    for chord in range(len(real_pb)):
        if real_pb[idx] == 'rep':
            labels['rep'] = [idx,0,1]
        elif real_pb[idx] == 'other':
            labels['other'] = [idx,0,1]
        elif real_pb[idx] == '*other':
            dom_column = final_pb.index('other')
            labels['*other'] = [idx,dom_column,2]
        elif real_pb[idx] == 'other*':
            dom_column = labels['other'][0]
            labels['other*'] = [idx,dom_column,2]         
        elif real_pb[idx] == 'norm':
            labels['norm'] = [idx,0,1]
        elif real_pb[idx] == '*norm':
            dom_column = final_pb.index('norm')
            labels['*norm'] = [idx,dom_column,2]      
        elif real_pb[idx] == 'norm*':
            dom_column = labels['norm'][0]
            labels['norm*'] = [idx,dom_column,2]       
        #if type(real_pb[chord]) != float:
        idx +=1

    pb_idx = [chord for chord in final_pb if type(chord)!= float]
    pb_idx = len(pb_idx)
    real_cs = [real_chord 
               for real_chord in final_cs 
               if type(real_chord) != float]
    idx = real_cs.index('last')
    for chord in range(len(real_cs)):    
        new_idx = idx+pb_idx
        if real_cs[idx] == 'last':
            labels['last'] = [new_idx,0,1]       
        elif real_cs[idx] == 'dom':
            dom_column = labels['last'][0]
            labels['dom'] = [new_idx,dom_column,2]     
        elif real_cs[idx] == 'sub':
            dom_column = labels['dom'][0]
            labels['sub'] = [new_idx,dom_column,3]   
        elif real_cs[idx] == 'cad64':
            dom_column = labels['dom'][0]
            labels['cad64'] = [new_idx,dom_column,3]           
        elif real_cs[idx] == 'sub*':
            dom_column = labels['sub'][0]
            labels['sub*'] = [new_idx,dom_column,4]     
        elif real_cs[idx] == 'sub**':
            dom_column = labels['sub*'][0]
            labels['sub**'] = [new_idx,dom_column,5]     
        elif real_cs[idx] == 'sub***':
            dom_column = labels['sub**'][0]
            labels['sub***'] = [new_idx,dom_column,6]         
        #if type(real_cs[chord]) != float:
        idx -= 1

    # Empty matrix
    num_chords = len([real_chord 
                      for real_chord in pb_chords+cs_chords
                     if type(real_chord) != float])
    matrix = [[None]*num_chords for n in range(num_chords)]
        
    # Populating the matrix
    for chord in final_pb+final_cs:
        if type(chord) != float:
            matrix[labels[chord][0]][labels[chord][1]] = labels[chord][2]
          
    return matrix
 
def surface_parameters(pb_chords, 
                       cs_chords, 
                       key_label, 
                       pb_final_melody, 
                       cs_final_melody, 
                       final_pb, 
                       final_cs):
    
    
    
    chord_labels = [chord 
                for chord in pb_chords+cs_chords]
    keys = [key_label]*len(chord_labels)
    sd = []
    mnh = []
    cnh = [0]*len(chord_labels)
    inv = []
    root_position = ['first', 'norm', 'dom', 'last']
    all_chords = pb_chords+cs_chords
    all_chords = [real_chord
                 for real_chord in all_chords
                 if type(real_chord) != float]
    final_chords = final_pb+final_cs
    final_chords = [real_chord
                   for real_chord in final_chords
                   if type(real_chord) != float]
    final_melody = pb_final_melody+cs_final_melody
    if key_label[0].islower():
        dominating = 'i/i'
    else:
        dominating = 'I/I'
        
    for idx in range(len(all_chords)):

        # chordal data
        (current_chordal_notes, 
         current_scale, 
         current_melodic_range) = chord_data(key_label, 
                                             all_chords[idx], 
                                             possible_octaves)    
        
        root_idx = current_scale.index(current_chordal_notes[0])
        seventh_idx = (root_idx+6)%7
        seventh = current_scale[seventh_idx]
        
        # dominating data
        (dom_chordal_notes, 
         dom_scale, 
         dom_melodic_range) = chord_data(key_label, 
                                             dominating, 
                                             possible_octaves)    
                

        # scale degree
        mel_note = final_melody[idx][0]
        if (mel_note[:-1] == current_chordal_notes[1] 
            or mel_note[:-1] == current_chordal_notes[2]):
            sd.append(1)
        else:
            sd.append(0)
            
        # melody non-harmonic
        non_harm = 0
        for n in final_melody[idx]:
            if (n[:-1] == seventh 
                and '7' not in all_chords[idx]):
                non_harm += 1
            elif n[:-1] not in current_chordal_notes:
                if n in dom_melodic_range:
                    non_harm += 3
                else:
                    non_harm += 4
                    
        mnh.append(non_harm)
               
        # inversion
        if final_chords[idx] in root_position:
            inv.append(0)
        else:
            inv.append(2)
            
    return keys, chord_labels, sd, mnh, cnh, inv

def theme_data(folder, theme_name):
    data_file = folder+'/'+theme_name+'_mel.csv'
    with open(data_file) as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=',')
        data = [row for row in csv_reader]
    key = data[0][0]
    tempo = int(data[1][0])
    time_sig = data[2][0]
    melody = [ast.literal_eval(item) for item in data[3]]
    rhythm = [ast.literal_eval(item) for item in  data[4]]
    chord_seq = data[5]    
    
    data_file = folder+'/'+theme_name+'_harm.csv'
    df = pandas.read_csv(data_file)
    chord_labels = list(df['chord function'])
    keys_labels = list(df['key'])    
    data_file = folder+'/'+theme_name+'_matrix.csv'
    with open(data_file) as csv_file:
        csv_reader = csv.reader(csv_file, delimiter=',')
        matrix = [row for row in csv_reader]
    matrix_by_columns = [[matrix[num_rows][num_col] 
                          for num_rows in range(len(chord_labels))] 
                         for num_col in range(len(chord_labels))]

    flat_matrix = [item 
                   for sublist in matrix 
                   for item in sublist]
    flat_matrix = [int(item) 
                   if len(item) > 0 
                   else item 
                   for item in flat_matrix]
    max_sub = max([item 
                   for item in flat_matrix 
                   if type(item) == int])
    theme_inheritance = [None]*len(chord_labels)
    sub_0_idx = [row 
                 for row in range(len(matrix)) 
                 if str(0) in matrix[row]][0]
    theme_inheritance[sub_0_idx] = 0
    
    dominating_idxs =[sub_0_idx]
    for sub_order in range(1, max_sub+1):
        new_dom_idxs = []
        for dom_idx in dominating_idxs:
            for chord_idx in range(len(chord_labels)):
                if matrix_by_columns[dom_idx][chord_idx] == str(sub_order):
                    inheritance = distance(keys_labels[dom_idx],
                                           chord_labels[dom_idx],
                                           chord_labels[chord_idx])[0]
                    theme_inheritance[chord_idx] = inheritance
                    new_dom_idxs.append(chord_idx)
        dominating_idxs = new_dom_idxs
        
    return (key, 
            tempo, 
            time_sig, 
            melody, 
            rhythm, 
            chord_seq, 
            chord_labels, 
            keys_labels, 
            theme_inheritance)        

def read_data(inpt, folder):
    
    (key, 
     tempo, 
     time_sig, 
     melody, 
     rhythm, 
     chord_seq, 
     chord_labels, 
     keys_labels, 
     theme_inheritance) = theme_data(folder, inpt)

    flat_melodies = [item for sublist in melody for item in sublist]
    flat_rhythms = [item for sublist in rhythm for item in sublist]

    div, subdiv = time_sig.split('/')
    if int(subdiv) == 4:
        reference_duration = 1.0
    else:
        reference_duration = .5

    rhythm_by_bars = []
    rhythm_in_bar = []
    melody_by_bars = []
    melody_in_bar = []
    bar_dur = 0.0
    bars_in_span = []
    bar_in_span = 1
    for ch in chord_seq[1:]:
        if ch == 'nan':
            bar_in_span += 1
        else:
            bars_in_span.append(bar_in_span)
            bar_in_span = 1
    bars_in_span.append(bar_in_span)        
    span_counter = 0
    for span_idx in range(len(flat_rhythms)):
        bar_dur += flat_rhythms[span_idx]
        melody_in_bar.append(flat_melodies[span_idx])
        rhythm_in_bar.append(flat_rhythms[span_idx])
        if bar_dur > int(div)*reference_duration*bars_in_span[span_counter]:
            rhythm_by_bars.append(rhythm_in_bar)
            melody_by_bars.append(melody_in_bar)
            rhythm_in_bar = []
            melody_in_bar = []
            bar_dur = bar_dur - int(div)*reference_duration   
            span_counter += 1
        elif bar_dur == int(div)*reference_duration*bars_in_span[span_counter]:
            rhythm_by_bars.append(rhythm_in_bar)
            melody_by_bars.append(melody_in_bar)
            rhythm_in_bar = []
            melody_in_bar = []
            bar_dur = 0.0
            span_counter += 1

    themes_data = {'tempo': tempo, 
                  'time_sig': time_sig, 
                  'chord_seq':chord_seq, 
                  'chord_labels': chord_labels, 
                  'keys_labels': keys_labels,
                  'flat_melody': flat_melodies,
                  'flat_rhythm': flat_rhythms,
                  'melody_by_bars': melody_by_bars,
                  'rhythm_by_bars': rhythm_by_bars}
    return themes_data

def mtt(theme_name, folder):
    attractions = a_calculator(folder+'/'+theme_name+'_arrangement.csv', 
                              folder+'/'+theme_name+'_harm.csv')+[0.0]
    tensions = t_calculator(folder+'/'+theme_name+'_harm.csv', 
                           folder+'/'+theme_name+'_matrix.csv')
            
    return tensions, attractions